/*
	
	Example of basic vector/axes/matrix operations available in Kray SDK utilities
		by Grzegorz Tanski

*/

#include "../kraysdk/type/vector.h"
#include "../kraysdk/type/axes.h"
#include "../kraysdk/math/matrix.h"
#include "../kraysdk/stdc/iostream.h"

#ifdef KRAY_USE_IOSTREAM_H
#include <iostream.h>
#else
#include <iostream>
#endif

int main(int argc, const char** argv){
	double matrix4Data[]={
							-0.9460853588275451	,0.3239174181981501	,0	,785.9417185662107,
							-0.3239174181981501	,-0.9460853588275451,0	,1146.907564734805,
							0					,0					,1	,0.1341845723732273,
							0					,0					,0	,1
						};


	Kray::Matrix4x4 matrix4;		// homogenous coordinates, 4x4 matrix that defines rotation, scaling and position
	matrix4.copyTransposed(matrix4Data);

	{
		// matrix4x4 object can be converted to Kray::Vector and Kray::Axes
		// it can be used wherever Kray::Axes or Kray::Vector is required (also with implicit conversion)

		std::cout << "Position             : " << (Kray::Vector)matrix4	<< std::endl;
		std::cout << "Rotation and scaling : " << (Kray::AxesHpb)matrix4	<< std::endl;

		std::cout << std::endl;
	}

	{
		// we can easily get rotation, scale and position from 4x4 matrix
		Kray::Vector position(matrix4);
		std::cout << "Position X : " << position[0] << std::endl; 
		std::cout << "Position Y : " << position[1] << std::endl; 
		std::cout << "Position Z : " << position[2] << std::endl; 

		Kray::AxesHpb axes(matrix4);
		std::cout << "Rotation H : " << axes.hpb(0) << " radians (Y axis, order 3)" << std::endl;		// rotation unit is radian
		std::cout << "Rotation P : " << axes.hpb(1) << " radians (X axis, order 2)" << std::endl;
		std::cout << "Rotation B : " << axes.hpb(2) << " radians (Z axis, order 1)" << std::endl;

		// HPB rotation angles, rotates object in YXZ axis respectively in BPH order
		// see Matrix3x3::fromAxes() and Matrix3x3::toAxes() in KraySDK source code for transformation details

		std::cout << "Scale    X : " << axes.scl(0) << std::endl;
		std::cout << "Scale    Y : " << axes.scl(1) << std::endl;
		std::cout << "Scale    Z : " << axes.scl(2) << std::endl;
	}
}