/* 

	Debugging tools demonstration. Check Debug and Release configurations for different results
		by Grzegorz Tanski

*/

#include <iostream>

#include "../kraysdk/kray.h"
#include "../kraysdk/buffer/binary.h"
#include "../kraysdk/buffer/text.h"
#include "../kraysdk/proto/direct.h"

#include "../SharedSources/libraryDir.h"

void functionThatCausesError(Kray::Prototyper& prot){
	prot.include("nonExistingFile.kray");
}

#ifdef _DEBUG
typedef Kray::CommandTextBuffer CommandBufferBaseClass;		// text, easy to read, but slow
#else
typedef Kray::CommandBinaryBuffer CommandBufferBaseClass;	// binary, hard to read, but fast
#endif

class MyException{
public:
	int code;

public:
	MyException(int code):code(code){
	}
};

class MyErrorHandlingCommandBuffer : public CommandBufferBaseClass{
public:

	MyErrorHandlingCommandBuffer(Kray::Instance* kinst):CommandBufferBaseClass(kinst){
	}

	void errorHandle(int code){
		throw MyException(code);
	}
};

int kraySdkExampleMain(int argc, const char** argv){
	Kray::Library klib(KRAY_DIRECTORY);
	Kray::Instance* kinst=klib.createInstance();
	
	if (kinst){
		std::cout << "=== Kray::CommandBinaryBuffer" << std::endl;

		{
			Kray::CommandBinaryBuffer tbuf(kinst);
			Kray::DirectPrototyper prot(tbuf);

			functionThatCausesError(prot);

			if (prot.buffer().lastError()){
				std::cout << std::endl;
				std::cout << "==! Kray::CommandBinaryBuffer error code=" << prot.buffer().lastError() << std::endl;

				prot.buffer().clearError();	// enable further parsing
			}
		}

		std::cout << std::endl << std::endl;

		std::cout << "=== Kray::CommandTextBuffer" << std::endl;

		{
			Kray::CommandTextBuffer tbuf(kinst);
			Kray::DirectPrototyper prot(tbuf);

			functionThatCausesError(prot);

			if (prot.buffer().lastError()){
				std::cout << std::endl;
				std::cout << "==! Kray::CommandTextBuffer error code=" << prot.buffer().lastError() << std::endl;
			}
		}

		std::cout << std::endl << std::endl;

		std::cout << "=== MyErrorHandlingCommandBuffer" << std::endl;

		try{
			MyErrorHandlingCommandBuffer tbuf(kinst);
			Kray::DirectPrototyper prot(tbuf);

			functionThatCausesError(prot);
		}
		catch(const MyException& e){
			std::cout << std::endl << "==! Kray parser caused an error. Code=" << e.code << std::endl;
		}

		std::cout << std::endl << std::endl;

	}else{
		std::cout << "Failed to load Kray instance" << std::endl;
	}

	return 0;
}