/*
	
	How to use Kray SDK to generate Kray script file, that can be rendered with Kray standalone.
	Script is generated OFFLINE (without loading Kray library), but the syntax is the same as for library calls.
		by Grzegorz Tanski

*/

#include <iostream>
#include <fstream>

#include "../kraysdk/kray.h"
#include "../kraysdk/proto/streams.h"

#include "../SharedSources/libraryDir.h"
#include "../SharedSources/basicMesh.h"

#define SCRIPTFILENAME PROJECT_DIRECTORY "ExportScriptToFile/basicMesh.kray"

void setupSimpleExportScene(Kray::Prototyper& prot){
	setupSimpleMeshScene(prot);

	// example how to mix direct script code with prototypes
	// calling Kray::Prototyper::parse() is best way to do it, such code is always correcly exported
	// direct call of Kray::Instance::parse() will not export script to file
	prot.parse("echo \"Note: \"\"mixing\"\" prototypes and direct script is possible.\";");

	// note how " are *escaped*
	// Kray script string
	// "this is ""quoting"" test"
	// is parsed as 
	// this is "quoting" test

	// it works the same for "string" and 'string'
	// and you can use
	// 'this is "quoting" test'
	// and
	// "this is 'quoting' test"
}

int main(int argc, const char** argv){
	// Note that we do not dynamically link with Kray in this example, only static KrayLib is used

	std::cout << "Offline Kray script generating example. Kray dynamic library NOT loaded" << std::endl;

	std::cout << std::endl<< "StdOutPrintCommandBuffer with packEnums=false" << std::endl << std::endl;

	{
		Kray::OstreamLinePrintPrototyper prot(std::cout,false);

		setupSimpleExportScene(prot);
	}

	std::cout << std::endl<< "StdOutPrintCommandBuffer with packEnums=true" << std::endl << std::endl;

	{
		Kray::OstreamLinePrintPrototyper prot(std::cout,true);

		setupSimpleExportScene(prot);
	}

	std::cout << std::endl<< "Saving script to file with OstreamCommandBuffer ..." << std::endl;

	{
		std::ofstream out(SCRIPTFILENAME);

		Kray::OstreamPrototyper prot(out,true);

		prot.parseMode(1);	// turns on, turbo parsing mode

		setupSimpleExportScene(prot);

		prot.render();	// add render commands to get some result when executed with Kray standalone
		prot.pause(120);
	}

	std::cout << "... file '" SCRIPTFILENAME "' saved" << std::endl << std::endl;
	std::cout << "One may try to run Kray standalone (executable) with '" SCRIPTFILENAME "' as parameter" << std::endl;
}