/*
	
	How to build a simple objects hierarchy, setup scene and render.
		by Grzegorz Tanski

*/

#include <iostream>

#include "../kraysdk/kray.h"
#include "../kraysdk/proto/direct.h"
#include "../kraysdk/type/vector.h"
#include "../kraysdk/type/axes.h"
#include "../kraysdk/type/coordsys.h"
#include "../kraysdk/symbol/mesh.h"
#include "../kraysdk/math/matrix.h"

#include "../SharedSources/basicMesh.h"
#include "../SharedSources/libraryDir.h"

int kraySdkExampleMain(int argc, const char** argv){	// main function is in startup.cpp
	Kray::Library klib(KRAY_DIRECTORY);				// here we start with KrayLib object
	Kray::Instance* kinst=klib.createInstance();	// then we create Kray instance, each instance have its own scene and settings
	
	if (kinst){									// check if instance was created, if not, Kray library wasn't found
		Kray::DirectPrototyper prot(kinst);		// prototyper gives access to Kray Script command prototypes

// build a hierarchy on scene
		Kray::MeshSymbol msh(prot);				// create a mesh
		buildBoxMesh(prot,msh);					// call box build function
		setupSimpleMaterialsForMesh(prot,msh);	// call material creating function

		if (1){	// copy(1) or instance(0)
			prot.animOb_typeMesh("mesh",msh);
		}else{
			prot.animOb_typeMeshInstance("mesh",msh);
		}

		prot.animOb_setActive("mesh",0);

		prot.animOb_typeCopy("m0","mesh");
		prot.animOb_setParent("m0","root");
		prot.animOb_setStatic("m0",Kray::CoordSys(Kray::Vector(0,0,0),Kray::AxesHpb().scale(0.2)));

		prot.animOb_typeCopy("m1","mesh");
		prot.animOb_setParent("m1","root");
		prot.animOb_setStatic("m1",Kray::CoordSys(Kray::Vector(-1,-1,0),Kray::AxesHpb().scale(0.4)));

		prot.animOb_typeCopy("m2","mesh");
		prot.animOb_setParent("m2","root");
		prot.animOb_setStatic("m2",Kray::CoordSys(Kray::Vector(1,-1,0),Kray::AxesHpb().scale(0.4)));

		prot.animOb_typeCopy("m3","mesh");
		prot.animOb_setParent("m3","root");
		prot.animOb_setStatic("m3",Kray::CoordSys(Kray::Vector(1,1,0),Kray::AxesHpb().scale(0.4)));

		prot.animOb_typeCopy("m4","mesh");
		prot.animOb_setParent("m4","root");
		prot.animOb_setStatic("m4",Kray::CoordSys(Kray::Vector(-1,1,0),Kray::AxesHpb().scale(0.4)));
		
		prot.animOb_typeNull("root");
		prot.animOb_setStatic("root",Kray::CoordSys(Kray::Vector(0,0,5),Kray::AxesHpb().degrees(12,33,44).scale(2)));

		prot.lightAdd_point(Kray::Vector(-1,-1,-2),Kray::Vector(1,1,1));			// add light source to scene
		prot.background_directionsMap();

		prot.showInfo_anim();
		prot.anim_setupFrame(0);

		prot.echo("Launching flayer....");
		prot.pv_flayer();
	}else{
		std::cout << "Failed to create Kray instance" << std::endl;
	}

	return 0;
}
