#ifndef SDK_EVENT_H
#define SDK_EVENT_H

#include "commons.h"

// text flags
#define KRAY_TEXT_GET_INTERRUPT		1
#define KRAY_TEXT_GET_UNICODE		2

// window flags
#define KRAY_WINDOW_GET_INTERRUPT	1
#define KRAY_WINDOW_GET_FOCUS		2

#define KRAY_WINDOW_SET_MOUSE_CAPTURE	4

// window resize flags
#define KRAY_WINDOW_ALPHA		1
#define KRAY_WINDOW_RESIZABLE	2
#define KRAY_WINDOW_NOCLOSE		4
#define KRAY_WINDOW_NOBORDERS	8

typedef enum _KrayEventType{
// draw
	KrayEventSetPixelArray,
	KrayEventSetPixelRectangle,

// text stream (log)
	KrayEventCreateTextStream,
	KrayEventDestroyTextStream,
	KrayEventText,
	KrayEventNewLine,
	KrayEventGetText,
	KrayEventGetTextProperty,

// window (preview)
	KrayEventCreateWindow,
	KrayEventDestroyWindow,

	KrayEventResizeWindow,
	KrayEventSetWindowTitle,

	KrayEventSetWindowProperty,
	KrayEventGetWindowProperty,
	KrayEventGetWindowMousePosition,
	KrayEventGetWindowMouseRawPosition,
	KrayEventGetWindowKeyEvent,

// common
	KrayEventAllocTextBuffer,		// allocates text buffer (optional, called from getTextLine() when textBufferSize==0)
	KrayEventFreeTextBuffer,

//	terminator
	KrayEventLast
} KrayEventType;

typedef enum _KrayWindowProperties{
	KrayWindowPropertyFlags,
	KrayWindowPropertyRawMouseDPI,
} KrayWindowProperties;

typedef enum _KrayTextProperties{
	KrayTextPropertyFlags,
} KrayTextProperties;

typedef struct _KrayEventPixel{
	int x;
	int y;

	int c;
} KrayEventPixel;

typedef struct _KrayEventPixelTable{
	KrayEventPixel* pxls;

	size_t count;
} KrayEventPixelTable;

typedef struct _KrayEventPixelRectangle{
	unsigned int* pixels;

	int modulo;
	int x1;
	int y1;
	int x2;
	int y2;
} KrayEventPixelRectangle;

typedef struct _KrayEventWindow{
	int positionX;
	int positionY;

	int width;
	int height;

	int flags;
} KrayEventWindow;

typedef struct _KrayEventKey{
	int keyCode;
	unsigned int miliSeconds;
	char type;
	char pressed;
} KrayEventKey;

typedef struct _KrayEventMousePosition{
	int x;
	int y;
} KrayEventMousePosition;

typedef struct _KrayEventTextBufferData{
	char* buffer;			// pointer to chars
	size_t bufferSize;		// size of chars space

	void* bufferControlData;	// user struct for buffer allocation (optional if buffer is not constant size)
								// non zero value of this field indicates, that buffer needs to be freed with
								// KrayFreeTextBuffer event

} KrayEventTextBufferData;

typedef struct _KrayEventData{
	KrayEventType type;
	KrayInstanceHandle instance;
	void* userData;

	union{
		void* windowHandle;		// frame handle for frame drawing operations
		void* textStreamHandle;	// text stream handle for text operations (log)
	};

	union{
		KrayBitmapHandle bitmap;

		const KrayEventPixelTable* pixelTable;
		const KrayEventPixelRectangle* pixelRectangle;
		const KrayEventWindow* window;

		KrayEventMousePosition* mousePosition;
		KrayEventKey* key;

		KrayEventTextBufferData* textBuffer;	// text output, const size or resizable buffer for chars (getTextLine)

		size_t propertyValue;				// interupted, unicode support of text input/output
	};

	union{
		const char* text;						// text input
		int propertyType;
	};
} KrayEventData;

#ifdef __cplusplus

namespace Kray{

typedef KrayEventType EventType;
typedef KrayWindowProperties WindowProperties;
typedef KrayTextProperties TextProperties;

class EventData : public KrayEventData{
};
class EventPixel : public KrayEventPixel{
};
class EventPixelTable : public KrayEventPixelTable{
};
class EventPixelRectangle : public KrayEventPixelRectangle{
};
class EventWindow : public KrayEventWindow{
};
class EventKey : public KrayEventKey{
};
class EventMousePosition : public KrayEventMousePosition{
};

};

#endif

#endif