#ifndef SDK_EVENT_LISTENER_H
#define SDK_EVENT_LISTENER_H

#ifndef __cplusplus
#error C++ required
#endif

#include "event.h"
#include "object.h"

namespace Kray{

class EventListener : public Object{
public:
// activation
	virtual void activate(class Instance* k){
	}	// called when listener is activated (with setEventListner() call)
	virtual void deactivate(class Instance* k){
	}	// called when listener is deactivated

// text log
	// overloading createTextStream and destroyTextStream is optional
	virtual void* createTextStream(){
		return this;	// default value of 0 would mean that we dont want textLine() textNewLine() notifications at all
	}
	virtual void destroyTextStream(void* handle){
	}

	virtual void textLine(void* handle,const char* txt){
	};	// display text line
	virtual void textNewLine(void* handle){
	};	// moves cursor to next line
	virtual void getTextLine(void* handle,const char* promptString,KrayEventTextBufferData* bufferData){
		bufferData->buffer=0;
	}
	virtual int getTextFlags(void* handle){
		return 0;
	}

// getLine buffer handling, overload only when custom buffer handling is needed
	virtual void allocTextBuffer(KrayEventTextBufferData* bufferData);
	virtual void freeTextBuffer(KrayEventTextBufferData* bufferData);
	
	bool textSupportsUnicode(void* handle){
		return (getTextFlags(handle)&KRAY_TEXT_GET_UNICODE)!=0;
	}
	bool textInterrupt(void* handle){	// does user wish to interupt current process?
		return (getTextFlags(handle)&KRAY_TEXT_GET_INTERRUPT)!=0;
	}

// gfx windows (preview)
	virtual void* createWindow(){
		return 0;
	}
	virtual void destroyWindow(void* windowHandle){
	}

	virtual int setWindowSize(void* windowHandle,const KrayEventWindow& kew){
		return 0;	// zero, failed, >0 ok
	};
	virtual void setPixels(void* windowHandle,const KrayEventPixelTable& kept){
	}
	virtual void copyPixels(void* windowHandle,const KrayEventPixelRectangle& kepr){
	}
	
	virtual int getWindowFlags(void* windowHandle){
		return 0;
	}
	virtual void setWindowFlags(void* windowHandle,int flags){
	}
	virtual void setWindowTitle(void* windowHandle,const char* title){
	}
	virtual void getWindowMouseRawPosition(void* windowHandle,KrayEventMousePosition& mp){	// mouse traced outside window, relative coordinates
		getWindowMousePosition(windowHandle,mp);
	}
	virtual void getWindowMousePosition(void* windowHandle,KrayEventMousePosition& mp);			// mouse traced inside window, absolute coordinates
	virtual void getWindowKeyEvent(void* windowHandle,KrayEventKey& key);
	virtual int getMouseRawDPI(void* windowHandle){
		return 0;
	}

	bool windowClosePressed(void* windowHandle){
		return (getWindowFlags(windowHandle)&KRAY_WINDOW_GET_INTERRUPT)!=0;
	}
	bool windowHasFocus(void* windowHandle){
		return (getWindowFlags(windowHandle)&KRAY_WINDOW_GET_FOCUS)!=0;
	}

// static tools
	static int getRedChannel(int c){
		return (c>>16)&0xff;
	}
	static int getGreenChannel(int c){
		return (c>>8)&0xff;
	}
	static int getBlueChannel(int c){
		return c&0xff;
	}
	static int getAlphaChannel(int c){
		return c>>24;
	}
};

class InstanceLinkedListener : public EventListener{		// this listner holds link to Instance and provides some additional utilities
protected:
	const Instance* kray;

public:
	InstanceLinkedListener():kray(0){
	}

	void activate(Instance* k){
		kray=k;
	}
	void deactivate(Instance* k){
		kray=0;
	}

	bool isEventString(int krayEventStringType,const char* string) const;	// type is one of KRAY_STREAM_EVENT_STRING constants
	int recognizeEventString(const char* string) const;						// return type is one of KRAY_STREAM_EVENT_STRING constants or KRAY_STREAM_EVENT_UNKNOWN
};

class BounceBackListener : public InstanceLinkedListener{
public:
	void* createTextStream();
	void destroyTextStream(void* h);
	int getTextFlags(void* handle);
	void textLine(void* handle,const char* txt);
	void textNewLine(void* handle);
	void getTextLine(void* handle,const char* promptString,KrayEventTextBufferData* bufferData);

	void* createWindow();
	void destroyWindow(void* windowHandle);

	int setWindowSize(void* windowHandle,const KrayEventWindow& kew);
	void setPixels(void* windowHandle,const KrayEventPixelTable& kept);
	void copyPixels(void* windowHandle,const KrayEventPixelRectangle &kepr);	

	int getMouseRawDPI(void* windowHandle);
	int getWindowFlags(void* windowHandle);
	void setWindowFlags(void* windowHandle,int flags);

	void setWindowTitle(void* windowHandle,const char* title);

	void getWindowMouseRawPosition(void* windowHandle,KrayEventMousePosition& mp);
	void getWindowMousePosition(void* windowHandle,KrayEventMousePosition& mp);
	void getWindowKeyEvent(void* windowHandle,KrayEventKey& key);
};

};

#endif