#ifndef SDK_MULTITHREADING_TOOLS_H
#define SDK_MULTITHREADING_TOOLS_H

#include "calling.h"
#include "object.h"

class KrayLibraryInstance;

namespace Kray{

typedef void (KRAYDLL_CALL_TYPE *ThreadCallbackFunction)(void* userData);

enum SyncObjectType{
	KrayMutexType
};

class MTGroup{
};

struct MTFunctions{
	static int KRAYDLL_CALL_TYPE getNumberOfCores();

	static void KRAYDLL_CALL_TYPE multiThreadedExecute(MTGroup* group,const size_t jobSize[3],int threads,void (KRAYDLL_CALL_TYPE *doJob)(const size_t jobIndex[3],int threadIndex,void *param),void *param);

	static MTGroup* KRAYDLL_CALL_TYPE allocGroup();
	static void KRAYDLL_CALL_TYPE freeGroup(MTGroup*);

	static void* KRAYDLL_CALL_TYPE create(SyncObjectType objectType,int p1,int p2);
	static char KRAYDLL_CALL_TYPE lock(void*,int lockCount,int p1);
	static void KRAYDLL_CALL_TYPE destroy(void*);

	static void KRAYDLL_CALL_TYPE setThreadCallbacks(KrayLibraryInstance* inst,ThreadCallbackFunction initThread,ThreadCallbackFunction exitThread,void* userData);
};

class MTFunctionsVtable : public Object{
public:
	int (KRAYDLL_CALL_TYPE *getNumberOfCores)();

	void (KRAYDLL_CALL_TYPE *multiThreadedExecute)(MTGroup* group,const size_t jobSize[3],int threads,void (KRAYDLL_CALL_TYPE *doJob)(const size_t jobIndex[3],int threadIndex,void *param),void *param);

	MTGroup* (KRAYDLL_CALL_TYPE *allocGroup)();
	void (KRAYDLL_CALL_TYPE *freeGroup)(MTGroup*);

	void* (KRAYDLL_CALL_TYPE *create)(SyncObjectType objectType,int p1,int p2);
	char (KRAYDLL_CALL_TYPE *lock)(void*,int lockCount,int p1);
	void (KRAYDLL_CALL_TYPE *destroy)(void*);

	void (KRAYDLL_CALL_TYPE *setThreadCallbacks)(KrayLibraryInstance* inst,ThreadCallbackFunction initThread,ThreadCallbackFunction exitThread,void* userData);

	void init();
};

class MTJobSplit : public Object{
protected:
	const MTFunctionsVtable* vt;
	MTGroup* group;

private:
	static void KRAYDLL_CALL_TYPE callback(const size_t jobIndex[3],int threadIndex,void *param){
		MTJobSplit* k=(MTJobSplit*)param;
		
		k->run(jobIndex,threadIndex);
	}
public:
	MTJobSplit(const MTFunctionsVtable* vt=0);	// when vt==0, class's API is still working but on a single thread
	~MTJobSplit();

	void start(const size_t jobSize[3],int threads);
	void start(int threads,size_t job0,size_t job1=1,size_t job2=1){
		size_t job[3];

		job[0]=job0;
		job[1]=job1;
		job[2]=job2;
		
		start(job,threads);
	}
	
	virtual void run(const size_t jobIndex[3],int thread){
	}
};

class Mutex{
	const MTFunctionsVtable* vt;
	void* mtx;
public:
	Mutex(const MTFunctionsVtable *vt=0){
		this->vt=vt;
		
		if (vt){
			mtx=vt->create(KrayMutexType,0,0);
		}else{
			mtx=0;
		}
	}
	~Mutex(){
		if (vt){
			vt->destroy(mtx);
		}
	}

	void lock(){
		if (vt){
			vt->lock(mtx,1,0);
		}
	}
	void unlock(){
		if (vt){
			vt->lock(mtx,-1,0);
		}
	}
	char trylock(){
		if (vt){
			return vt->lock(mtx,0,0);
		}
		
		return 1;
	}
};

};

#endif