#ifndef SDK_DIRECT_RAYTRACE_COPY_HELPERS_H
#define SDK_DIRECT_RAYTRACE_COPY_HELPERS_H

#include "mt.h"

namespace Kray{

class RaytraceProcessorUtilities{
public:
	template <class A> static void computeBucketJob(size_t bucket[3],const A& buffer,const size_t* job=0){
		if (!job){
			job=buffer.allocSize;
		}

		for (int a=0 ; a<3 ; a++){
			bucket[a]=(job[a]+buffer.bucketSize[a]-1)/buffer.bucketSize[a];
		}
	}
};

template <class P> class RaytraceProcessorBase : protected MTJobSplit, protected RaytraceProcessorUtilities{
protected:
	P& processor;
	size_t job[3];

protected:
	template <class A> void initShiftAndSize(const A& buffer,const size_t bucket[3],size_t shift[3],size_t size[3]){
		for (int a=0 ; a<3 ; a++){
			shift[a]=buffer.bucketSize[a]*bucket[a];
			size[a]=shift[a]+buffer.bucketSize[a];
			if (size[a]>job[a]){
				size[a]=job[a];
			}
			
			if (size[a]>shift[a]){
				size[a]-=shift[a];
			}else{
				size[a]=0;
			}
		}
	}

public:
	RaytraceProcessorBase(P& processor,const MTFunctionsVtable* vt=0):processor(processor),MTJobSplit(vt){
	}

	template <class A> void initProcess(const A& buffer,const size_t* job=0){
		if (!job){
			job=buffer.allocSize;
		}
		
		for (int a=0 ; a<3 ; a++){
			this->job[a]=job[a];
		}		
	}

	template <class A> void process(const A& buffer,int threads=0,const size_t* job=0){
		initProcess(buffer,job);

		size_t bucketJob[3];
		computeBucketJob(bucketJob,buffer,job);

		MTJobSplit::start(bucketJob,threads);
	}
};

template <class A,class P> class RaytraceAProcessor : public RaytraceProcessorBase<P>{
protected:
	typedef typename A::CellType CellType;

protected:
	const A& buffer;

public:
	RaytraceAProcessor(const A& buffer,P& processor,const MTFunctionsVtable* vt=0):buffer(buffer),RaytraceProcessorBase<P>(processor,vt){
	}

	void process(int threads=0,const size_t* job=0){	// this one creates threads and calls back run()
		RaytraceProcessorBase<P>::process(buffer,threads,job);
	}

	void run(const size_t bucket[3],int threadId){	// call this, only when you have own thread spawing and synchronization
		size_t shift[3];
		size_t size[3];

		this->initShiftAndSize(buffer,bucket,shift,size);
				
		CellType* dta=buffer.getBucketBuffer(bucket);
		
		size_t c[3];
		size_t job[3];
		for (c[2]=0 ; c[2]<size[2] ; c[2]++){
			job[2]=c[2]+shift[2];
			for (c[1]=0 ; c[1]<size[1] ; c[1]++){
				job[1]=c[1]+shift[1];
				for (c[0]=0 ; c[0]<size[0] ; c[0]++){
					job[0]=c[0]+shift[0];

					this->processor.process(dta[c[0]+size[0]*(c[1]+size[1]*c[2])],job);
				}
			}
		}
	}
};

template <class A,class B,class P> class RaytraceABProcessor : public RaytraceProcessorBase<P>{
protected:
	typedef typename A::CellType CellTypeA;
	typedef typename B::CellType CellTypeB;

protected:
	const A& aBuffer;
	const B& bBuffer;

public:
	RaytraceABProcessor(const A& buffer1,const B& buffer2,P& processor,const MTFunctionsVtable* vt=0):aBuffer(buffer1),bBuffer(buffer2),RaytraceProcessorBase<P>(processor,vt){
	}

	void process(int threads=0,const size_t* job=0){	// this one creates threads and calls back run()
		RaytraceProcessorBase<P>::process(aBuffer,threads,job);
	}

	void run(const size_t bucket[3],int threadId){	// call this, when you have own thread spawing and synchronization
		size_t shift[3];
		size_t size[3];

		this->initShiftAndSize(aBuffer,bucket,shift,size);
				
		CellTypeA* aDta=aBuffer.getBucketBuffer(bucket);
		CellTypeB* bDta=bBuffer.getBucketBuffer(bucket);
		
		size_t c[3];
		size_t job[3];
		for (c[2]=0 ; c[2]<size[2] ; c[2]++){
			job[2]=c[2]+shift[2];
			for (c[1]=0 ; c[1]<size[1] ; c[1]++){
				job[1]=c[1]+shift[1];
				for (c[0]=0 ; c[0]<size[0] ; c[0]++){
					job[0]=c[0]+shift[0];

					size_t i=c[0]+size[0]*(c[1]+size[1]*c[2]);

					this->processor.process(aDta[i],bDta[i],job);
				}
			}
		}
	}
};

};

#endif