#ifndef KRAY_TEXT_TOOLS_H
#define KRAY_TEXT_TOOLS_H

#include "cmd.h"
#include "../utils/textbuffer.h"

namespace Kray{

// human readable and streamable (text) CommandBuffer, allows expressions
class CommandTextBuffer : public CommandBuffer{
protected:
	TextBuffer tb;

	bool packEnums;		// compress Kray enum names or not
public:
	CommandTextBuffer(Instance* kinst,bool packEnums=false):CommandBuffer(kinst),packEnums(packEnums){
	}
	
	void addEnumQuick(int index);

	// CommandBuffer interface
	void generateAutoname(class Symbol& s,char name[kraySymbolNameMaxChars]);

	void addEnum(int index,const char *name);

	void addDouble(double v){
		tb + v;

		setArgumentSeparator();
	}
	void addString(const char* v);
	void addVector(const class Vector& v);
	void addAxes(const class Axes& v);
	void addCoordSys(const class CoordSys& v);
	void addSymbol(const class Symbol& v);
	void addVarLenDouble(const class VarLenDouble& varlen);
	void addVarLenVector(const class VarLenVector& varlen);
	void addVarLenAxes(const class VarLenAxes& varlen);
	void addVarLenCoordSys(const class VarLenCoordSys& varlen);
	void addVarLenSymbol(const class VarLenSymbol& varlen);
	
	const char* toString(){
		return tb;
	}
	void parse();
	
	// Native methods
	
	TextBuffer& textBuffer(){
		return tb;
	}
	void setPackEnums(bool a){
		packEnums=a;
	}
protected:
	template <class T> void addVarLen(const VarLenTemplate<T>& varlen);
	const char* getTerminatedBuffer();

	void addAxesCore(const struct KrayAxesCore& v);

	void setArgumentSeparator();
	void setCommandSeparator();
};

// streamable (text) CommandBuffer, harder to read for humans but faster then CommandTextBuffer, does not allow expressions (all params explicit)
class CommandQuickTextBuffer : public CommandTextBuffer{	
public:
	CommandQuickTextBuffer(Instance* kinst):CommandTextBuffer(kinst,1){
	}

	void addEnum(int index,const char* name){
		addEnumQuick(index);
	}

	void parse();
};

};

#endif