#include "../kraysdk/kray.h"
#include "../kraysdk/base/event.h"
#include "../kraysdk/base/listener.h"
#include "../kraysdk/base/event.h"
#include "../kraysdk/base/mt.h"
#include "../kraysdk/base/bmp.h"
#include "../kraysdk/base/bmpcall.h"
#include "../kraysdk/base/rt.h"
#include "../kraysdk/base/prscall.h"
#include "../kraysdk/base/getinfo.h"
#include "../kraysdk/buffer/dynamic.h"
#include "../kraysdk/utils/str2num.h"
#include "../kraysdk/base/dispatchlistener.h"

using namespace Kray;

char Vtables::init(){
//	if (ok()) return 1;

/*	if (!(kcore=new CoreFunctionsVtable)){		// move allocation to CORE side
		exit();
		return 0;
	}
	if (!(kmt=new MTFunctionsVtable)){
		exit();
		return 0;
	}
	if (!(kbmp=new BitmapFunctionsVtable)){
		exit();
		return 0;
	}
	if (!(krt=new RaytraceFunctionsVtable)){
		exit();
		return 0;
	}
	if (!(kprs=new ParserFunctionsVtable)){
		exit();
		return 0;
	}*/
	
	return 1;
}
void Vtables::exit(){
//	delete kmt;				// move deletion to CORE side
	kmt=0;
	
//	delete kbmp;
	kbmp=0;
	
//	delete krt;
	krt=0;
	
//	delete kprs;
	kprs=0;

	if (kcore){
		kcore->exit();

//		delete kcore;
		kcore=0;
	}
}

////////////////////////////////////////

Library::Library(KrayLibInitFunctionPointer dlli,const char* dir):flags(flagDefault),libraryUserData(0){
	zero();

	if (!dir){
		dir=DLLCURRENTDIR;
	}

	setLibraryDir(dir);

	initLib(dlli);
}
Library::Library(const char* dir):flags(flagDefault){
	zero();

	if (!dir){
		dir=DLLCURRENTDIR;
	}

	setLibraryDir(dir);
}
Library::~Library(){
	release();

	setLibraryDir(0);
}
void Library::zero(){
	libraryDir=0;
	hLib=0;
	firstInstance=0;
	hostApiVersion=KRAY_API_VERSION;
	krayModules=0;
}
void Library::setLibraryDir(const char* dir){
	freeTable(libraryDir);
	libraryDir=0;

	if (dir){
		if (dir[0]){
			char* dirCopy=allocateTable<char>(s2n_strlen(dir)+1);

			if (dirCopy){
				s2n_strcpy(dirCopy,dir);

				libraryDir=dirCopy;
			}
		}
	}
}
bool Library::initLib(void* hLib){
	if (!hLib){
		return 0;
	}	

	return initLib((KrayLibInitFunctionPointer)DllFindProcedure(hLib, KrayLibInitFunctionString));
}

bool Library::initLib(KrayLibInitFunctionPointer dlli){
	if (!dlli){
		return 0;
	}

	if (!vts.init()){
		release();

		return 0;
	}

	dlli(this);

	if (!isLoaded()){
		release();

		return 0;
	}

	return 1;
}
bool Library::load(const char* dir){
	if (isLoaded()){
		return 1;
	}

	if (dir){
		setLibraryDir(dir);
	}

	hLib=DllLoadDebugOrReleaseVersion(libraryDir,KrayLibBaseName,DLLDEFAULTEXTENSION);

	if (!hLib){
		return 0;
	}

	return initLib(hLib);
}
bool Library::hasInstances() const{
	return firstInstance!=0;
}
bool Library::isLoaded() const{
	return getLibApiVersion()!=0;
}
void Library::release(){
	if (!getFlag(flagDontReleaseInstances) && firstInstance){	// release all instances that was not released yet
		flags|=flagLibraryIsBeingDestructed;

		Instance* k=firstInstance;
		firstInstance=0;	// disable instances list
		
		while(k){
			Instance* kn=k->nextInstance;
			delete k;
			k=kn;
		};

		flags&=~flagLibraryIsBeingDestructed;
	}
	
	if (getFlag(flagUnloadIfEmpty)){
		vts.exit();

		if (hLib){
			DllUnload(&hLib);
			hLib=0;
			krayModules=0;
		}
	}
}
Instance* Library::allocInstance(){
	if (!isLoaded()){
		if (!load()){
			return 0;
		}
	}
	
	return new Instance(this);
}
Instance* Library::initInstance(Instance* i){
	KrayLibraryInstance* kli=vtable().kcore->createInstance(i);
			
	if (!kli){
		delete i;
		
		return 0;
	}

	i->nextInstance=firstInstance;
	firstInstance=i;	

	return i;
}
Instance* Library::createInstance(){
	Instance* i=allocInstance();
	if (!i) return 0;
	
	i->setDefaultEventListener();

	return initInstance(i);
}
Instance* Library::createInstanceWithListner(EventListener* listner){
	Instance* i=allocInstance();
	if (!i) return 0;
	
	i->setEventListener(listner);

	return initInstance(i);	
}
Instance* Library::createInstanceWithHandler(EventHandler handler,void* userData){
	Instance* i=allocInstance();
	if (!i) return 0;
	
	i->setEventHandler(handler,userData);

	return initInstance(i);
}
void Library::releaseInstance(Instance* i){
	delete i;
}
void Library::unregister(Instance* k){
	Instance** link=&firstInstance;
	
	while(*link){
		if (*link==k){
			*link=k->nextInstance;
			return;
		}
		
		*link=(*link)->nextInstance;
	}

	if (!hasInstances()){
		release();
	}
}
int Library::setPlatformOption(const char* option,const char* stringData,void* binaryData) const{
	if (vts.kcore){
		return vts.kcore->setPlatformOption(option,stringData,binaryData);
	}else{
		return 0;
	}	
}
const char* Library::getInfoString(int mode) const{
	if (vts.kcore){
		return vts.kcore->getKrayString(mode);
	}else{
		return 0;
	}
}
const char* Library::getLibraryDir() const{
	return libraryDir;
}

int Library::getInfoInt(int mode) const{
	if (vts.kcore){
//		if (vts.kcore->ok()){
		return vts.kcore->getKrayFlag(mode);
//		}
	}

	return 0;
}
int Library::getLibApiVersion() const{	// returns 0, when lib is not loaded
	return getInfoInt(KRAY_INT_API_VERSION);
}

////////////////////////////////////////////////////////////////////

Instance::Instance(Library* parent):kinst(0),parent(parent),defaultListener(0),listenerObject(0),nextInstance(0){
	kcore=parent->vtable().kcore;
}
Instance::~Instance(){
	vtableCore()->releaseInstance(kinst);

	setEventListener();
	
	delete defaultListener;
	parent->unregister(this);	
}

////////////////////////////////////////////////////////////////////

void KRAYDLL_CALL_TYPE kray_null_handler(EventData* ked){
}
void KRAYDLL_CALL_TYPE kray_cpp_handler(EventData* ked){
	EventListener* kel=((Instance*)ked->instance)->getListener();
	
	switch(ked->type){
		default:
		break;
		
		case KrayEventSetPixelArray:
			kel->setPixels(ked->windowHandle,*ked->pixelTable);
		break;
		case KrayEventSetPixelRectangle:
			kel->copyPixels(ked->windowHandle,*ked->pixelRectangle);
		break;

		case KrayEventCreateWindow:
			ked->windowHandle=kel->createWindow();
		break;
		case KrayEventDestroyWindow:
			kel->destroyWindow(ked->windowHandle);
		break;
		case KrayEventGetWindowProperty:
			switch(ked->propertyType){
				case KrayWindowPropertyFlags:
					ked->propertyValue=	(kel->windowClosePressed(ked->windowHandle)?	KRAY_WINDOW_GET_INTERRUPT	:0)|
										(kel->windowHasFocus(ked->windowHandle)?		KRAY_WINDOW_GET_FOCUS		:0);
				break;
				case KrayWindowPropertyRawMouseDPI:
					ked->propertyValue= kel->getMouseRawDPI(ked->windowHandle);
				break;
			}
		break;
		case KrayEventSetWindowProperty:
			switch(ked->propertyType){
				case KrayWindowPropertyFlags:
					kel->setWindowFlags(ked->windowHandle,(int)ked->propertyValue);
				break;
			}
		break;
		case KrayEventResizeWindow:
			if (!kel->setWindowSize(ked->windowHandle,*ked->window)){
				ked->windowHandle=0;
			}
		break;
		case KrayEventSetWindowTitle:
			kel->setWindowTitle(ked->windowHandle,ked->text);
		break;
		case KrayEventGetWindowMouseRawPosition:
			kel->getWindowMouseRawPosition(ked->windowHandle,*ked->mousePosition);
		break;
		case KrayEventGetWindowMousePosition:
			kel->getWindowMousePosition(ked->windowHandle,*ked->mousePosition);
		break;
		case KrayEventGetWindowKeyEvent:
			kel->getWindowKeyEvent(ked->windowHandle,*ked->key);
		break;

		case KrayEventCreateTextStream:
			ked->textStreamHandle=kel->createTextStream();
		break;
		case KrayEventDestroyTextStream:
			kel->destroyTextStream(ked->textStreamHandle);
		break;
		case KrayEventNewLine:
			kel->textNewLine(ked->textStreamHandle);
		break;
		case KrayEventText:
			kel->textLine(ked->textStreamHandle,ked->text);
		break;
		case KrayEventGetText:
			kel->getTextLine(ked->textStreamHandle,ked->text,ked->textBuffer);
		break;
		case KrayEventAllocTextBuffer:
			kel->allocTextBuffer(ked->textBuffer);
		break;
		case KrayEventFreeTextBuffer:
			kel->freeTextBuffer(ked->textBuffer);
		break;

		case KrayEventGetTextProperty:
			ked->propertyValue=kel->getTextFlags(ked->textStreamHandle);
		break;
	}
}
void Instance::setEventHandler(void (KRAYDLL_CALL_TYPE* handler)(EventData*),void* userData){
	setEventListener();

	this->handler=handler;
	this->listenerObject=0;
	this->userData=userData;
}
void Instance::setEventListener(EventListener* eventListener){
	if (listenerObject){
		getListener()->deactivate(this);
	}
	if (eventListener){
		handler=kray_cpp_handler;
		listenerObject=eventListener;

		eventListener->activate(this);
	}else{
		handler=kray_null_handler;
		listenerObject=0;
	}
}
void Instance::setDefaultEventListener(){
	if (!defaultListener){
		defaultListener=new BounceBackListener;
	}

	setEventListener(defaultListener);
}
EventListener* Instance::getEventListener() const{
	if (listenerObject==defaultListener) return 0;

	return listenerObject;
}

////////////////////////////////////////////////////////////////////////////////

void EventListener::allocTextBuffer(KrayEventTextBufferData* bufferData){
	// alloc buffer, set its adress and set data that needs to be released after buffer is read
	bufferData->bufferControlData=bufferData->buffer=allocateTable<char>(bufferData->bufferSize);
}
void EventListener::freeTextBuffer(KrayEventTextBufferData* bufferData){
	freeTable((char*)bufferData->bufferControlData);	// when buffer is too small to fit data, allocTextBuffer() allocs dymanic buffer that needs to be freed here
	bufferData->bufferControlData=0;					// when bufferControlData is already zero, this line and line above does nothing
}

void EventListener::getWindowMousePosition(void* windowHandle,KrayEventMousePosition& mp){
	mp.x=0;
	mp.y=0;
}
void EventListener::getWindowKeyEvent(void* windowHandle,KrayEventKey& key){
	key.type=0;
}

////////////////////////////////////////////////////////////////////

bool InstanceLinkedListener::isEventString(int krayEventStringType,const char* string) const{
	if (!kray || !string) return false;

	const char* eventString=kray->getKrayLib()->getInfoString(krayEventStringType);

	if (!eventString) return false;

	while(*string && *eventString){
		if (*string++!=*eventString++) return false;
	}

	return true;
}
int InstanceLinkedListener::recognizeEventString(const char* string) const{
	for (int a=KRAY_STREAM_EVENT_STRING ; a<KRAY_STREAM_EVENT_END ; a++){
		if (isEventString(a,string)){
			return a;
		}
	}

	return KRAY_STREAM_EVENT_UNKNOWN;
}

////////////////////////////////////////////////////////////////////

void BounceBackListener::getTextLine(void* handle,const char* prompt,KrayEventTextBufferData* bufferData){
	EventData ked;
	ked.type=KrayEventGetText;
	ked.text=prompt;
	ked.textBuffer=bufferData;
	ked.textStreamHandle=handle;
	
	kray->sendBackEvent(ked);
}
void BounceBackListener::textLine(void* handle,const char* txt){
	EventData ked;
	ked.type=KrayEventText;
	ked.text=txt;
	ked.textStreamHandle=handle;
	
	kray->sendBackEvent(ked);
}
void BounceBackListener::textNewLine(void* handle){
	EventData ked;
	ked.type=KrayEventNewLine;
	ked.textStreamHandle=handle;
	
	kray->sendBackEvent(ked);
}
int BounceBackListener::getTextFlags(void* handle){
	EventData ked;
	ked.type=KrayEventGetTextProperty;
	ked.propertyType=KrayTextPropertyFlags;
	ked.textStreamHandle=handle;

	kray->sendBackEvent(ked);

	return (int)ked.propertyValue;
}
void* BounceBackListener::createTextStream(){
	EventData ked;

	ked.type=KrayEventCreateTextStream;
	kray->sendBackEvent(ked);

	return ked.textStreamHandle;
}
void BounceBackListener::destroyTextStream(void* handle){
	EventData ked;

	ked.type=KrayEventDestroyTextStream;
	ked.textStreamHandle=handle;

	kray->sendBackEvent(ked);
}

int BounceBackListener::setWindowSize(void* windowHandle,const KrayEventWindow& kew){
	EventData ked;
	ked.type=KrayEventResizeWindow;	
	ked.window=&kew;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);

	return ked.windowHandle!=0;
}
void BounceBackListener::setPixels(void* windowHandle,const KrayEventPixelTable& kept){
	EventData ked;
	ked.type=KrayEventSetPixelArray;	
	ked.pixelTable=&kept;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);
}
void BounceBackListener::copyPixels(void* windowHandle,const KrayEventPixelRectangle& kepr){
	EventData ked;
	ked.type=KrayEventSetPixelRectangle;	
	ked.pixelRectangle=&kepr;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);
}
void* BounceBackListener::createWindow(){
	EventData ked;
	ked.type=KrayEventCreateWindow;

	kray->sendBackEvent(ked);
	
	return ked.windowHandle;
}
void BounceBackListener::destroyWindow(void* windowHandle){
	EventData ked;
	ked.type=KrayEventDestroyWindow;
	
	ked.windowHandle=windowHandle;
	
	kray->sendBackEvent(ked);	
}

void BounceBackListener::setWindowTitle(void* windowHandle,const char* title){
	EventData ked;
	ked.type=KrayEventSetWindowTitle;	
	ked.windowHandle=windowHandle;
	ked.text=title;

	kray->sendBackEvent(ked);	
}

int BounceBackListener::getMouseRawDPI(void* windowHandle){
	EventData ked;
	ked.type=KrayEventGetWindowProperty;
	ked.propertyType=KrayWindowPropertyRawMouseDPI;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);

	return (int)ked.propertyValue;
}
int BounceBackListener::getWindowFlags(void* windowHandle){
	EventData ked;
	ked.type=KrayEventGetWindowProperty;
	ked.propertyType=KrayWindowPropertyFlags;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);

	return (int)ked.propertyValue;
}
void BounceBackListener::setWindowFlags(void* windowHandle,int flags){
	EventData ked;
	ked.type=KrayEventSetWindowProperty;	
	ked.propertyType=KrayWindowPropertyFlags;
	ked.windowHandle=windowHandle;

	ked.propertyValue=flags;
	kray->sendBackEvent(ked);
}
void BounceBackListener::getWindowMousePosition(void* windowHandle,KrayEventMousePosition& mp){
	EventData ked;
	ked.type=KrayEventGetWindowMousePosition;
	ked.mousePosition=&mp;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);
}
void BounceBackListener::getWindowMouseRawPosition(void* windowHandle,KrayEventMousePosition& mp){
	EventData ked;
	ked.type=KrayEventGetWindowMouseRawPosition;
	ked.mousePosition=&mp;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);
}
void BounceBackListener::getWindowKeyEvent(void* windowHandle,KrayEventKey& key){
	EventData ked;
	ked.type=KrayEventGetWindowKeyEvent;
	ked.key=&key;
	ked.windowHandle=windowHandle;

	kray->sendBackEvent(ked);
}

///////////////////////////////////////////////

void DispatchLinkedListener::destroyTextStream(void* handle){
	delete (TextStreamListener*)handle;
}
