#if defined(KRAY_LIB_STARTUP) || defined(KRAY_LIB_STARTUP_WIDE_CHAR)

#include <stddef.h>
#include <stdio.h>

#include "../kraysdk/memhandler.h"
#include "../kraysdk/staticlink.h"
#include "../kraysdk/startup.h"
#include "../kraysdk/kray.h"
#include "../kraysdk/buffer/dynamic.h"
#include "../kraysdk/buffer/binary.h"
#include "../kraysdk/commands.h"
#include "../kraysdk/startup.h"
#include "../kraysdk/base/listener.h"
#include "../kraysdk/proto/proto.h"

void loadLibraryFromFilename(Kray::Library& klib,const char* rsrcFile){
	if (rsrcFile){
		size_t len=0;
		size_t slash=0;

		while(rsrcFile[len]!=0){
			if (rsrcFile[len]=='/' || rsrcFile[len]=='\\'){
				slash=len+1;
			}

			len++;
		}

		ResizableBuffer rsrcDir;

		rsrcDir.append(rsrcFile,slash);
		rsrcDir.append(0);

		klib.load(rsrcDir.getBuffer());
	}
}

int krayLibraryStartup(int argc, const char** argv){
	Kray::Library& klib=Kray::getLibrary();

	ResizableBuffer args;

/*
#ifdef __APPLE__
	if (!isCocoaStartup(argc,argv)){
		args.appendString(" -pstdio");
	}
#endif
*/
	
	for (int a=1 ; a<argc ; a++){
		if (argv[a][0]=='-'){
			if (argv[a][1]=='l'){
				a++;

				if (a<argc){
					klib.load(argv[a]);
				}

				continue;
			}
		}

		args.append(' ');

		args.append('"');
		args.appendString(argv[a]);			
		args.append('"');
	}
		
	args.append(0);
	
#ifdef KRAY_LIB_COCOA_BUNDLE
	loadLibraryFromFilename(klib,getCocoaResource(KrayLibBaseName DLLDEFAULTEXTENSION));
#endif

	klib.load();	// try load -l option or current dir

	if (!klib.isLoaded()){
		// try loading from same dir where executable file is
		ResizableBuffer dir;
		dir.appendString(argv[0]);
		dir.append(0);

		// find last slash or backslash and truncate filename after it
		char* t=dir.getBuffer();

		char* filePart=t;

		while(*t){
			if (*t=='/' || *t=='\\'){
				filePart=t+1;
			}

			t++;
		}

		*filePart=0;	// cut off file part, leave directory path only

		klib.load(dir.getBuffer());
	}

	////////////////////////////////////////////

	Kray::Instance* kinst=klib.createInstanceWithListner(0);

	if (!kinst){
		fputs("!!! Unable to load Kray library (use -l krayLibDir/ option)\n",stdout);

#ifdef WIN32
		{
			fputs("    Press enter to continue\n",stdout);
			fflush(stdout);

			char c[5];
			fgets(c,sizeof(c)-1,stdin);
		}
#endif

		return -1;
	}

	Kray::CommandBinaryBuffer tbuf(kinst);

	(tbuf + KrayCmd(kray) + args.getBuffer() + 0x100).parse();

	klib.releaseInstance(kinst);

	return 0;
}

#ifdef KRAY_LIB_STARTUP
int main(int argc, const char** argv){
	return krayCallMain(krayLibraryStartup,argc,argv);
}
#elif defined(KRAY_LIB_STARTUP_WIDE_CHAR)
#include "../kraysdk/utils/argsutf8.h"

int wmain(int argc,const wchar_t** argv){
	CommandLineUTF8Encoder<wchar_t> mae(argc,argv);
	
	return krayCallMain(krayLibraryStartup,mae.argc(),mae.argv());
}
#endif

#endif