#include "../kraysdk/memhandler.h"
#include "../kraysdk/base/parser.h"
#include "../kraysdk/base/commons.h"
#include "../kraysdk/base/prscall.h"

#include "../kraysdk/buffer/text.h"
#include "../kraysdk/buffer/binary.h"
#include "../kraysdk/buffer/offline.h"
#include "../kraysdk/type/double.h"
#include "../kraysdk/type/vector.h"
#include "../kraysdk/type/axes.h"
#include "../kraysdk/type/coordsys.h"
#include "../kraysdk/utils/utf8.h"
#include "../kraysdk/utils/str2num.h"
#include "../kraysdk/kray.h"

using namespace Kray;

void CommandBuffer::addString(const wchar_t *v){
	addString(UTF8EncoderBuffer<wchar_t>(v));
}
CommandBuffer& CommandBuffer::operator+(const AxesHpb& v){
	addAxes(v);

	return *this;
}
KrayScriptParser* CommandBuffer::getParser(){
	return kinst->getDefaultParser();
}
void CommandBuffer::error(int code){
	errorCode=code;

	if (code){
		errorHandle(code);
	}
}
char CommandBuffer::encode(int i){
	const int l1=10;
	const int l2='z'-'a';
//	const int l3='Z'-'A';

	if (i<l1) return i+'0';
	if (i<(l1+l2)) return i-l1+'a';

	return i-l1-l2+'A';
}
void CommandBuffer::encode(size_t v,char* name,int radix){
	int len=0;
	int first=0;
	size_t vv=v;

	while(vv>0){
		first=(int)vv;
		len++;
		vv/=radix;
	}

	if (first<10){
		*name++='_';
	}

	for (int a=len-1 ; a>=0 ; a--){
		name[a]=encode((int)(v%radix));

		v/=radix;
	}
	
	name[len]=0;
}
void CommandBuffer::generateAutoname(Symbol& s,char name[kraySymbolNameMaxChars]){
	encode((size_t)(void*)&s,name);
}

/////////////////////////////////////////////////////////////

void CommandTextBuffer::setCommandSeparator(){
	if (packEnums){
		tb.setSeparate(",");
	}else{
		tb.setSeparateIf(","," ");
	}
}
void CommandTextBuffer::setArgumentSeparator(){
	tb.setSeparate(",");
}

void CommandTextBuffer::generateAutoname(Symbol& s,char name[kraySymbolNameMaxChars]){
	if (packEnums){
		encode((size_t)(void*)&s,name);
	}else{
		encode((size_t)(void*)&s,name,16);	// generated names will be simply hex adresses
	}
}
void CommandTextBuffer::addString(const char* v){
	tb += v;

	setArgumentSeparator();
}
void CommandTextBuffer::addVector(const class Vector& v){
	tb.separate();
	tb.concat("(");
	tb.concat(v.v[0]);
	for (int a=1 ; a<v.count ; a++){
		tb.concat(",");
		tb.concat(v.v[a]);		
	}
	tb.concat(")");

	setArgumentSeparator();
}
void CommandTextBuffer::addCoordSys(const CoordSys& v){
	tb.separate();
	tb.concat("(");
	tb.concat(v.v[0]);
	for (int a=1 ; a<krayParserAxesMax ; a++){
		tb.concat(",");
		tb.concat(v.v[a]);		
	}
	tb.concat(")");

	tb.concat("@");

	addAxesCore(v.a);
}
void CommandTextBuffer::addAxes(const class Axes& v3){
	tb.separate();

	addAxesCore(v3);
}
void CommandTextBuffer::addAxesCore(const KrayAxesCore& v){
	Axes v3(v);

	if (v3.isEye()){
		tb.concat("<>");
	}else if (v3.isAxesHpb()){
		AxesHpb v;
		v3.isAxesHpb(v);

		tb.concat("<");

		if (!v.isHpbNulls()){
			tb.concat(v.hpb(0)*180/PI);
			for (int a=1 ; a<v.dim() ; a++){
				tb.concat(",");
				tb.concat(v.hpb(a)*180/PI);		
			}
		}

		tb.concat(">");
	
		if (!v.isSclOnes()){
			tb.concat("*(");

			tb.concat(v.scl(0));

			if (!v.isSclProportional()){
				for (int a=1 ; a<v.dim() ; a++){
					tb.concat(",");
					tb.concat(v.scl(a));		
				}
			}
			tb.concat(")");
		}
	}else{
		tb.concat("axes(");

		for (int c=0 ; c<v3.count ; c++){
			tb.concat(c>0?",(":"(");

			for (int a=0 ; a<krayParserAxesMax ; a++){
				tb.concat(a>0?",":"");
				tb.concat(v3.matrix[c][a]);
			}

			tb.concat(")");
		}

		tb.concat(")");
	}

	setArgumentSeparator();
}
void CommandTextBuffer::addSymbol(const class Symbol& v){
	tb + v.getName();

	setArgumentSeparator();
}
template <class T> void CommandTextBuffer::addVarLen(const VarLenTemplate<T>& varlen){
	tb.separate();

	if (varlen.length()>1){
		tb.concat("{");
	}

	tb.setSeparate(0);

	for (size_t i=0 ; i<varlen.length() ; i++){
		(*this) + varlen[i];
	}

	if (varlen.length()>1){
		tb.concat("}");
	}

	setArgumentSeparator();
}
void CommandTextBuffer::addVarLenDouble(const VarLenDouble& v){
	addVarLen(v);
}
void CommandTextBuffer::addVarLenVector(const VarLenVector& v){
	addVarLen(v);
}
void CommandTextBuffer::addVarLenAxes(const VarLenAxes& v){
	addVarLen(v);
}
void CommandTextBuffer::addVarLenCoordSys(const VarLenCoordSys& v){
	addVarLen(v);
}
void CommandTextBuffer::addVarLenSymbol(const class VarLenSymbol& v){
	addVarLen(v);
}

void CommandTextBuffer::addEnumQuick(int index){
	tb.separate();
		
	tb.concat("[");
	tb.concat(index);
	tb.concat("]");
	
	setCommandSeparator();
}
void CommandTextBuffer::addEnum(int index,const char* name){
	if (packEnums || !name){
		addEnumQuick(index);
	}else{
		tb + name;

		tb.setSeparateIf(","," ");
	}
}
const char* CommandTextBuffer::getTerminatedBuffer(){
	tb.concat(";");
	tb.terminate();

	return toString();
}
void CommandTextBuffer::parse(){
	if (errorCode) return;

	error(kinst->parse(getTerminatedBuffer(),"CommandTextBuffer"));
}

///////////////////////////////////////////////////////////

void CommandQuickTextBuffer::parse(){
	if (errorCode) return;

	error(kinst->parseQuick(getTerminatedBuffer(),"CommandQuickTextBuffer"));
}

//////////////////////////////////////////////////////////////

void CommandBinaryBuffer::addString(const char* v){
	if (v){
		size_t len=s2n_strlen(v);
		appendId(len);
		bb.append(v,len+1);
	}else{
		size_t len=0;
		appendId(len);		
	}
}
void CommandBinaryBuffer::parse(){
	appendId((size_t)KrayParseSectionClosing);	// terminate whole script
	
	storeHeader();
	bb.truncate();
	
	if (errorCode) return;

	error(kinst->parseBinary(bb.getBuffer()));

	allocHeader();	
}
void CommandBinaryBuffer::addVector(const class Vector& v){
	appendId(v.count);
	
	for (int a=0 ; a<v.count ; a++){
		bb.appendBinary(v.v[a]);
	}
}
void CommandBinaryBuffer::addAxes(const Axes& v){
	addAxesCore(v);
}
void CommandBinaryBuffer::addAxesCore(const KrayAxesCore& v){
	appendId(v.count);
	
	for (int c=0 ; c<v.count ; c++){
		for (int a=0 ; a<krayParserAxesMax ; a++){
			bb.appendBinary(v.matrix[c][a]);
		}
	}
}
void CommandBinaryBuffer::addCoordSys(const class CoordSys& v){
	for (int a=0 ; a<krayParserAxesMax ; a++){
		bb.appendBinary(v.v[a]);
	}

	addAxesCore(v.a);
}
void CommandBinaryBuffer::addSymbol(const class Symbol& v){
	if (v.getSymbolHandle()){
		appendId(0);
		
		bb.appendBinary(v.getSymbolHandle());
	}else{
		addString(v.getName());
	}
}

template <class T> void CommandBinaryBuffer::addVarLen(const VarLenTemplate<T>& varlen){
	appendId((size_t)KrayParseSectionOpening);
	
	bb.appendBinary((size_t)0);
	size_t lenCursor=bb.cursor();

	for (size_t i=0 ; i<varlen.length() ; i++){
		(*this) + varlen[i];
	}

	appendId((size_t)KrayParseSectionClosing);

	bb.replaceBinary(lenCursor-sizeof(size_t),bb.cursor()-lenCursor);
}
void CommandBinaryBuffer::addVarLenDouble(const VarLenDouble& v){
	addVarLen(v);
}
void CommandBinaryBuffer::addVarLenVector(const VarLenVector& v){
	addVarLen(v);
}
void CommandBinaryBuffer::addVarLenAxes(const VarLenAxes& v){
	addVarLen(v);
}
void CommandBinaryBuffer::addVarLenCoordSys(const VarLenCoordSys& v){
	addVarLen(v);
}
void CommandBinaryBuffer::addVarLenSymbol(const VarLenSymbol& v){
	addVarLen(v);
}

///////////////////////////////////////////////////////////////////////////

void CommandOfflineBuffer::parse(){
	out(getTerminatedBuffer());
}
