#ifndef SDKCOMMONS_H
#define SDKCOMMONS_H

#include <stddef.h>	// for size_t, not used, by this header, but by many dependent headers

#define krayMaxMacro(a,b) ((a)>(b)?(a):(b))
#define krayMinMacro(a,b) ((a)<(b)?(a):(b))

#define KrayLibInitFunction			kray3_init
#define KrayLibInitFunctionString	"kray3_init"
#define KrayLibCoreName				"kray3"		// base and core can have the same name, since core uses CPU subtype suffixes
#define KrayLibBaseName				"kray3"

#define KRAY_API_VERSION				3000

#define KRAY_ASYNC_PARSE_ON				1
#define KRAY_ASYNC_PARSE_OFF			0
#define KRAY_ASYNC_PARSE_WAIT			-1
#define KRAY_ASYNC_PARSE_FORGET			-2
#define KRAY_ASYNC_PARSE_SET_SIZE		-3
#define KRAY_ASYNC_PARSE_PAUSE			-4

//#define KRAY_ADD_FLAG_SINGLESIDED		0x1
#define KRAY_ADD_FLAG_NEEDUV			0x2		// do we use UV mapping on this polygon?
#define KRAY_ADD_FLAG_SMOOTHING			0x4		// shading normals needs to be computed
#define KRAY_ADD_FLAG_DISPLACE			0x8		// displace vectors needs to be computed
#define KRAY_ADD_FLAG_INVISIBLE			0x10	// do not add geometry to scene (but add LEM if used)
#define KRAY_ADD_FLAG_DOUBLESIDED		0x20	// do we want double sided polygon?

#define KRAY_ADD_FLAG_RECTANGLE_LEMS	0x1000	// find quad that makes a rectangle and add rectangle to the scene (useful for rectangle area lights)
//#define KRAY_ADD_FLAG_AUTO_GROUPS		0x2000
//#define KRAY_ADD_FLAG_AUTO_NORMALSIDE	0x4000
#define KRAY_ADD_FLAG_USE_SURFACE_MAP	0x2000  // use special surface/blend/clipmap switch (fast surface replace)
#define KRAY_ADD_FLAG_INSTANCE			0x8000	// set if your object will be instanced
#define KRAY_ADD_FLAG_EXPAND_MB_STATIC	0x10000	// use rendered faster but less mem friendly motion blur handeling of meshes
#define KRAY_ADD_FLAG_COMPRESS_NORMALS	0x20000	// use preprocessed, slower, but more mem friendly normals storage
#define KRAY_ADD_FLAG_NO_MB				0x40000 // do not use motion blur
#define KRAY_ADD_FLAG_FLIP_NORMALSIDE	0x80000	// change polygon facing to opposite

#define KRAY_NEEDBUFFER_BIT_RGB				0
#define KRAY_NEEDBUFFER_BIT_ALPH			1	// alpha channel
#define KRAY_NEEDBUFFER_BIT_TXTR			2	// texure
#define KRAY_NEEDBUFFER_BIT_ZBUF			3	// z buffer
#define KRAY_NEEDBUFFER_BIT_DIRE			4	// direct light
#define KRAY_NEEDBUFFER_BIT_SHDN			5	// shadow density
#define KRAY_NEEDBUFFER_BIT_INDI			6	// indirect illumination
#define KRAY_NEEDBUFFER_BIT_POSI			7	// ray hit position
#define KRAY_NEEDBUFFER_BIT_CAUS			8	// caustics
#define KRAY_NEEDBUFFER_BIT_REFL			10	// reflection
#define KRAY_NEEDBUFFER_BIT_REFR			12	// refraction
#define KRAY_NEEDBUFFER_BIT_LUMI			14	// luminosity
#define KRAY_NEEDBUFFER_BIT_OTHR			16	// other
#define KRAY_NEEDBUFFER_BIT_SFID			18	// surface id
#define KRAY_NEEDBUFFER_BIT_OBID			20	// object id
#define KRAY_NEEDBUFFER_BIT_NORM			22	// normal map

// TODO: wrap those below to Kray namespace 

typedef void* KrayHandle;

typedef void* KraySurfaceHandle;
typedef void* KrayBlendHandle;
typedef void* KrayMaterialHandle;
typedef void* KrayVertexMapHandle;
typedef void* KrayTextureHandle;
typedef void* KrayBitmapHandle;
typedef void* KrayMeshHandle;
typedef void* KrayObjectGroupHandle;

typedef void* KrayInstanceHandle;

typedef void* KrayObject;

typedef enum _KrayHandleType{
	KraySurfaceType,
	KrayBlendType,
	KrayMaterialType,
	KrayVertexMapType,
	KrayTextureType,
	KrayBitmapType,
	KrayMeshType,
	KrayObjectGroupType,
	KrayDtType,
	KraySymbolType,
} KrayHandleType;

#endif
