#ifndef SDK_PARSER_H
#define SDK_PARSER_H

#include <stddef.h>

#define krayParserVectorMax	4
#define krayParserAxesMax	3
#define kraySymbolNameMaxChars 128

typedef void* KrayDtHandle;

typedef struct _kraySymbolObject{ void* dummy; } *KraySymbolHandle;

struct KrayDoubleCore{
	double v;
};
struct KrayStringCore{
	const char* string;
};
struct KrayStringableBuffer{
	char* string;
	size_t size;		// size of string array with trailing 0, if buffer is too small, needed size is returned here (and string==0)
};
struct KrayVectorCore{
	double v[krayParserVectorMax];
	int count;
};
struct KrayAxesCore{
	double matrix[krayParserAxesMax][krayParserAxesMax];
	int count;		// can be 0 (eye axes) 1 (hpb) 2 (hpbscl) 3 (ax,ay,az)
};
struct KrayCoordSysCore{
	double v[krayParserAxesMax];		// position
	struct KrayAxesCore a;				// orientation
};
struct KraySymbolCore{
	char name[kraySymbolNameMaxChars];
	
	void* commandBufferHandle;
	
	KrayDtHandle directObjectHandle;
	KraySymbolHandle directSymbolHandle;
};

typedef enum _KrayDtTypes{
	KrayDtUnknownType=0,

	KrayDtStringType,			// does not work with parseExpression(), since string is temporary. Use KrayDtStringableType instead
	KrayDtDoubleType,
	KrayDtVectorType,
	KrayDtAxesType,
	KrayDtCoordSysType,

	KrayDtStringableType,		// convertible to string
	
	KrayDtSymbolType,
	
	KraySymbolCoreType,
	
	KrayDtEndOfBasicTypes	// last on this list
} KrayDtTypes;

typedef enum _KrayParseMark{	// all negative (complementary to command ids, and vector lengths), used in binary script mode
	KrayParseLineEnd=-1,
	KrayParseSectionOpening=-2,
	KrayParseSectionClosing=-3
} KrayParseMark;


#endif