#ifndef SDK_PARSER_CALL_H
#define SDK_PARSER_CALL_H

#include "calling.h"
#include "parser.h"
#include "object.h"

union KrayParserExchangeBufferUnion{
	struct KrayDoubleCore value;
	struct KrayStringCore string;
	struct KrayStringableBuffer stringable;
	struct KrayVectorCore vector;
	struct KrayAxesCore axes;
	struct KrayCoordSysCore coordsys;
};

struct KrayParserExchangeBuffer{
	union KrayParserExchangeBufferUnion u;

	KrayDtTypes dtType;
};

#define PARSE_ERR_OK				0
#define PARSE_ERR_BAD_CALL			1
#define PARSE_ERR_BAD_CAST			2
#define PARSE_ERR_INTERRUPTED		3	// user interupted parsing
#define PARSE_ERR_ASYNC_ADD_FAIL	4

#ifdef __cplusplus

class KrayScriptParser;

namespace Kray{

struct KrayParserFunctions{
	static KrayHandle KRAYDLL_CALL_TYPE getObjectByName(KrayScriptParser* ksp,const char* name,KraySymbolHandle* symbolHandle,KrayHandleType type,const char* context);
	static int KRAYDLL_CALL_TYPE parseScript(KrayScriptParser* ksp,const char* script,const char* script_name,int parse_mode);
	static int KRAYDLL_CALL_TYPE parseAsync(KrayScriptParser* ksp,int modeAsync,size_t param);
	static void KRAYDLL_CALL_TYPE showError(KrayScriptParser* ksp,const char* script,const char* script_name);

	static void KRAYDLL_CALL_TYPE getValue(KrayScriptParser* ksp,const KrayDtHandle dt,KrayParserExchangeBuffer* data);
	static void KRAYDLL_CALL_TYPE setValue(KrayScriptParser* ksp,const KrayDtHandle dt,const KrayParserExchangeBuffer* data);

	static size_t KRAYDLL_CALL_TYPE parseExpression(KrayScriptParser* ksp,const char* expression,KrayParserExchangeBuffer* data);
};

class ParserFunctionsVtable : public Object{
public:
	KrayHandle (KRAYDLL_CALL_TYPE *getObjectByName)(KrayScriptParser* ksp,const char* name,KraySymbolHandle* symbolHandle,KrayHandleType type,const char* context);
	int (KRAYDLL_CALL_TYPE *parseScript)(KrayScriptParser* ksp,const char* script,const char* script_name,int parse_mode);		
	void (KRAYDLL_CALL_TYPE *showError)(KrayScriptParser* ksp,const char* script,const char* script_name);	
	int (KRAYDLL_CALL_TYPE *parseAsync)(KrayScriptParser* ksp,int parseAsync,size_t param);	

	void (KRAYDLL_CALL_TYPE *getValue)(KrayScriptParser* ksp,const KrayDtHandle dt,KrayParserExchangeBuffer* data);
	void (KRAYDLL_CALL_TYPE *setValue)(KrayScriptParser* ksp,const KrayDtHandle dt,const KrayParserExchangeBuffer* data);

	size_t (KRAYDLL_CALL_TYPE *parseExpression)(KrayScriptParser* ksp,const char* expression,KrayParserExchangeBuffer* data);

	void init(){
		getObjectByName=&(KrayParserFunctions::getObjectByName);
		parseScript=&(KrayParserFunctions::parseScript);
		showError=&(KrayParserFunctions::showError);
		parseAsync=&(KrayParserFunctions::parseAsync);

		getValue=&(KrayParserFunctions::getValue);		
		setValue=&(KrayParserFunctions::setValue);
		parseExpression=&(KrayParserFunctions::parseExpression);
	}	
};

};
#endif

#endif