#ifndef KRAY_VAR_LEN_H
#define KRAY_VAR_LEN_H

#include <stddef.h>

namespace Kray{

class VarLen{		// variable length array of given type (template base)
protected:
	size_t count;

protected:
	VarLen(){
	}

public:
	size_t length() const{
		return count;
	}
};

template <class T> class VarLenTemplate : public VarLen{
protected:
	typedef T TYPE;

protected:
	T* v;

protected:
	VarLenTemplate(){
	}

public:
	T& operator[](size_t i){
		return v[i];
	}
	const T& operator[](size_t i) const{
		return v[i];
	}
};

template <class P,size_t N> class VarLenFixedTemplate : public P{
	typedef typename P::TYPE T;

	T fixed[N];

public:
	VarLenFixedTemplate(){
		P::v=fixed;
		P::count=N;
	}
};

template <class P> class VarLenDynamicTemplate : public P{
	typedef typename P::TYPE T;

	size_t allocated;

public:
	VarLenDynamicTemplate(size_t size=0){
		P::v=0;
		P::count=0;
		allocated=0;

		resize(size);
	}
	~VarLenDynamicTemplate(){
		delete P::v;
	}

	void set(const T& t0){
		setLength(1);

		P::v[0]=t0;
	}
	void set(const T& t0,const T& t1){
		setLength(2);

		P::v[0]=t0;
		P::v[1]=t1;
	}
	void set(const T& t0,const T& t1,const T& t2){
		setLength(3);

		P::v[0]=t0;
		P::v[1]=t1;
		P::v[2]=t2;
	}
	void set(const T& t0,const T& t1,const T& t2,const T& t3){
		setLength(4);

		P::v[0]=t0;
		P::v[1]=t1;
		P::v[2]=t2;
		P::v[3]=t3;
	}
	void set(const T& t0,const T& t1,const T& t2,const T& t3,const T& t4){	// in C++11 we would use variadic template or initializer list
		setLength(5);

		P::v[0]=t0;
		P::v[1]=t1;
		P::v[2]=t2;
		P::v[3]=t3;
		P::v[4]=t4;
	}

	T& operator[](size_t i){
		fit(i+1);

		return P::v[i];
	}
	const T& operator[](size_t i) const{
		return P::v[i];
	}

	void setLength(size_t a){
		fit(a);
		P::count=a;
	}
	void fit(size_t i){
		if (i>P::count){
			if (i>allocated){
				if (allocated*2>i){
					resize(allocated*2);
				}else{
					resize(i);
				}
			}

			P::count=i;
		}
	}
	void resize(size_t size){
		T* ndta=0;

		if (size>0){
			ndta=new T[size];
		}

		size_t limit=(size < P::count) ? size : P::count;

		for (size_t i=0 ; i<limit ; i++){
			ndta[i]=P::v[i];
		}

		delete P::v;
		P::v=ndta;

		allocated=size;
	}
};

};

#endif