#ifndef KRAY_BIN_TOOLS_H
#define KRAY_BIN_TOOLS_H

#include "../base/parser.h"
#include "../base/commons.h"
#include "../buffer/cmd.h"

#include "dynamic.h"

namespace Kray{

// not streamable CommandBuffer (binary with native endianess), not readable to human, but fastest
class CommandBinaryBuffer : public CommandBuffer{
private:
	DynamicBuffer bb;

	void appendId(size_t id){
		bb.appendBinary(id);
	}
	void allocHeader(){
		appendId(0);	// alloc header space
	}
	void storeHeader(){
		*((size_t*)bb.getBuffer())=bb.cursor();
	}
public:
	CommandBinaryBuffer(Instance* kinst):CommandBuffer(kinst){
		allocHeader();
	}
	
	// CommandBuffer interface
	void addEnum(int index,const char*){
		appendId(index);
	}

	void addDouble(double v){
		appendId(0);
		bb.appendBinary(v);
	}
	void addString(const char* v);
	void addVector(const class Vector& v);
	void addAxes(const class Axes& v);
	void addCoordSys(const class CoordSys& v);
	void addSymbol(const class Symbol& v);
	void addVarLenDouble(const class VarLenDouble& varlen);
	void addVarLenVector(const class VarLenVector& varlen);
	void addVarLenAxes(const class VarLenAxes& varlen);
	void addVarLenCoordSys(const class VarLenCoordSys& varlen);
	void addVarLenSymbol(const class VarLenSymbol& varlen);
		
	void parse();

protected:
	template <class T> void addVarLen(const VarLenTemplate<T>& varlen);
	void addAxesCore(const struct KrayAxesCore& v);
};

};

#endif