#ifndef KRAY_BUFFER_H
#define KRAY_BUFFER_H

#include "../base/varlen.h"
#include "../base/parser.h"
#include "../base/prscall.h"

namespace Kray{

class CommandBuffer{
protected:
	class Instance* kinst;
	int errorCode;
	
protected:
	static char encode(int v);
	static void encode(size_t v,char* name,int radix=(10+'z'-'a'+'Z'-'A'));

protected:
	CommandBuffer(Instance* kinst):errorCode(0),kinst(kinst){
	}

public:
	virtual ~CommandBuffer(){
	}

public:	
	CommandBuffer& buffer(){
		return *this;
	}
	class Instance* getKrayInstance() const{
		return kinst;
	}
	KrayScriptParser* getParser();

	int lastError() const{
		return errorCode;
	}
	void clearError(){
		error(0);
	}
	
	virtual void errorHandle(int code){	// overload to handle parsing errors
	}
	virtual void parse()=0;
	virtual void generateAutoname(class Symbol& s,char name[kraySymbolNameMaxChars]);

	virtual void addEnum(int index,const char* name)=0;
	virtual void addDouble(double v)=0;
	virtual void addString(const char* v)=0;
	virtual void addString(const wchar_t* v);
	virtual void addVector(const class Vector& v)=0;
	virtual void addAxes(const class Axes& v)=0;
	virtual void addCoordSys(const class CoordSys& v)=0;
	virtual void addSymbol(const class Symbol& v)=0;

	virtual void addVarLenDouble(const class VarLenDouble& varlen)=0;
	virtual void addVarLenVector(const class VarLenVector& varlen)=0;
	virtual void addVarLenAxes(const class VarLenAxes& varlen)=0;
	virtual void addVarLenCoordSys(const class VarLenCoordSys& varlen)=0;
	virtual void addVarLenSymbol(const class VarLenSymbol& varlen)=0;
			
	CommandBuffer& operator+(bool v){
		addDouble(v?1:0);
		return *this;
	}
	CommandBuffer& operator+(int v){
		addDouble((double)v);
		return *this;
	}
	CommandBuffer& operator+(float v){
		addDouble(v);
		return *this;
	}
	CommandBuffer& operator+(double v){
		addDouble(v);
		return *this;
	}
	CommandBuffer& operator+(size_t v){
		addDouble((double)v);
		return *this;
	}
	CommandBuffer& operator+(const char* v){
		addString(v);
		return *this;
	}
	CommandBuffer& operator+(char* v){
		addString(v);
		return *this;
	}
	CommandBuffer& operator+(const wchar_t* v){
		addString(v);
		return *this;
	}

	CommandBuffer& operator+(const Vector& v){
		addVector(v);
		return *this;
	}
	CommandBuffer& operator+(const Axes& v){
		addAxes(v);
		return *this;
	}
	CommandBuffer& operator+(const class AxesHpb& v);
	CommandBuffer& operator+(const CoordSys& v){
		addCoordSys(v);
		return *this;
	}
	CommandBuffer& operator+(const Symbol& v){
		addSymbol(v);
		return *this;
	}
	CommandBuffer& operator+(const Symbol* v){
		addSymbol(*v);
		return *this;
	}
	CommandBuffer& operator+(const class VarLenDouble& v){
		addVarLenDouble(v);
		return *this;
	}
	CommandBuffer& operator+(const class VarLenVector& v){
		addVarLenVector(v);
		return *this;
	}
	CommandBuffer& operator+(const class VarLenAxes& v){
		addVarLenAxes(v);
		return *this;
	}
	CommandBuffer& operator+(const class VarLenCoordSys& v){
		addVarLenCoordSys(v);
		return *this;
	}
	CommandBuffer& operator+(const class VarLenSymbol& v){
		addVarLenSymbol(v);
		return *this;
	}

	void error(int code);
};

};

#endif