#ifndef KRAY_1x_API_H
#define KRAY_1x_API_H

#include "../base/calling.h"
#include "../base/bmp.h"
#include "../base/vmap.h"
#include "../base/event.h"
#include "../base/getinfo.h"

#ifdef __cplusplus
#include "../kray.h"
#include "../base/listener.h"
#include "../base/addinfo.h"
#include "../base/bmpcall.h"
#include "../utils/resizablebuffer.h"
#endif

#define KRAY_RENDERASYNC		1
#define KrayShapeNodeType KrayObjectGroupType

#ifdef KRAY_NEW_API
#error Kray Deprecated 1.x C API and new C++ API cannot be used at the same time
#endif

#define KRAY_ADD_FLAG_SINGLESIDED 0		// deprecated

// old handle names (replaced by C++ classes)
#define KRAY_OLD_API
#define KraySurface KraySurfaceHandle
#define KrayBlend KrayBlendHandle
#define KrayMaterial KrayMaterialHandle
#define KrayVertexMap KrayVertexMapHandle
#define KrayTexture KrayTextureHandle
#define KrayBitmap KrayBitmapHandle
#define KrayMesh KrayMeshHandle
#define KrayObjectGroup KrayObjectGroupHandle
#define KrayShapeNode KrayObjectGroupHandle
#define KrayDt KrayDtHandle

#define Kray2RenderEventDone ((KrayEventType)((int)KrayEventLast+1))

#define KrayRenderEventDone _KrayRenderEventDone
#define KrayEventNewLine _KrayEventNewLine
#define KrayEventText _KrayEventText

typedef enum _KrayRenderEventType{
	_KrayRenderEventDone,KrayRenderEventFrameFinished,KrayEventSetPixel,KrayEventSetPixels,KrayEventGetUserInput,
	KrayEventWindowSize,_KrayEventNewLine,_KrayEventText
} KrayRenderEventType;

typedef enum _KrayUserResponse{
	KrayUserNoAction,KrayUserInterrupt
} KrayUserResponse;

typedef struct _KrayRenderEventData{
	const void *listener_param;
	
	union{
		KrayBitmap *bitmap;
		const char *text;

		const struct _KrayEventPixel *pixel;
		const struct _KrayEventPixels *pixels;
		const struct _KrayEventWindow *window;

		KrayUserResponse user;	// /*enum _KrayUserResponse*/
	};
	KrayRenderEventType type;
} KrayRenderEventData;

#ifdef __cplusplus

typedef void (KRAYDLL_CALL_TYPE *KrayRenderCallbackFunction)(Kray::EventData*);

class KrayLib1x : public Kray::Library{
	Kray::Instance* kray;
	Kray::EventListener* listener;
	KrayRenderCallbackFunction handler;
	KrayObjectGroupHandle defaultGroup;

	ResizableBuffer k2Cmd;		// for k2 parse mode

	char parseMode;		// 0 - k2 mode, 1 - native mode

public:
	KrayLib1x():kray(0),listener(0),handler(0),parseMode(0){
	}
	~KrayLib1x(){
		release();
	}
	
	void renderInit();
	void renderCleanUp();
	void render(int flags);	
	void renderReInit(){
		renderInit();
	}

	void setEventListener(Kray::EventListener* eventListener=0){
		listener=eventListener;
	}
	void setEventHandler(KrayRenderCallbackFunction handler){
		this->handler=handler;
	}
	KrayObject addTriangle(const float* points,const float* normals,const float* geometric_normal,KraySurfaceHandle surface,KrayBlendHandle blend,KrayTextureHandle clipmap,KrayObjectGroupHandle group,size_t vertex_map_poly_index,int flags){
		return kray->addTriangleDirect(Kray::TriangleAddInfo(	group?group:defaultGroup,
																	points,
																	surface,
																	blend,
																	flags,
																	normals,
																	clipmap,
																	vertex_map_poly_index,
																	geometric_normal));
	}
	int addPolyToMesh(KrayMeshHandle msh,int n,int sfid,size_t* indices,int group=0){
		return kray->addPolyToMesh(Kray::PolyAddInfo(msh,n,indices,sfid,group));
	}

	KrayHandle getObjectByName(const char* name,KrayHandleType type){
		return kray->getObjectByName(name,type);
	}
	void getBitmapInfo(const KrayBitmapHandle bitmap,KrayBitmapInfo* binfo){
		kray->vtableBitmap()->getBitmapInfo(bitmap,binfo);
	}
	
	int parse(const char* script,const char* name=0);
	
	const char* getKrayVersionString() const{
		return Library::getInfoString(KRAY_STRING_VERSION);
	}

	void sendBackEvent(Kray::EventData& ked) const{
		kray->sendBackEvent(ked);
	}
	
	Kray::Instance* instance() const{
		return kray;
	}
	
	KrayLib1x* operator->(){
		return this;
	}

	char load();
	void release();
};

class Kray2DeprecatedListner : public Kray::InstanceLinkedListener{		// wrapper to old Kray 2 interface
// wrapper
protected:
	int setWindowSize(void* windowHandle,const KrayEventWindow& kew){
		setWindowSize(kew.width,kew.height);		// redirect to deprecated method be default

		return 1;
	};
	void setPixels(void* windowHandle,const KrayEventPixelTable& kept){
		setPixels(kept);	// redirect to deprecated method
	}
	void copyPixels(void* windowHandle,const KrayEventPixelRectangle& kepr){
		copyPixels(kepr);	// redirect to deprecated method
	}

	void textLine(const char* txt,void* handle){
		textLine(txt);
	};
	void textNewLine(void* handle){
		textNewLine();
	};
	const char* getTextLine(const char* promptString,void* handle){
		return getTextLine(promptString);
	}
	virtual int getTextFlags(void* handle){
		return getTextFlags();
	}

	bool textSupportsUnicode(void* handle){
		return (getTextFlags(handle)&KRAY_TEXT_GET_UNICODE)!=0;
	}
	bool textInterrupt(void* handle){	// does user wish to interupt current process?
		return (getTextFlags(handle)&KRAY_TEXT_GET_INTERRUPT)!=0;
	}

// interface
public:
// deprecated interface methods (still works, but not recommended and does not support multiple windows)
	virtual void textLine(const char* txt){
	};
	virtual void textNewLine(){
	};
	virtual const char* getTextLine(const char* prompt_string){
		return 0;
	}
	virtual int getTextFlags(){
		return 0;
	}

	virtual void setPixels(const KrayEventPixelTable& kept);
	virtual void copyPixels(const KrayEventPixelRectangle& kepr){
		copyPixels(kepr.pixels,kepr.modulo,kepr.x1,kepr.y1,kepr.x2,kepr.y2);
	}
	virtual void setWindowSize(int x,int y,int flags=0){
	};
	virtual void setPixel(int x,int y,int c){
	};
	virtual void copyPixels(unsigned int* pixels,int sx,int x1,int y1,int x2,int y2);
	virtual bool interupt(){
		return false;
	}
	void newLine(){
		textNewLine();
	}
};

extern KrayLib1x kray;

#else
#include "kray1x_pure_c.h"
#endif	// __cplusplus

#endif