#ifndef KRAY_SYMBOL_H
#define KRAY_SYMBOL_H

#include "../base/parser.h"
#include "../base/commons.h"
#include "../base/varlen.h"

namespace Kray{

class DynamicBuffer;

class Symbol : protected KraySymbolCore{
protected:
	virtual void updateSymbol(){
	}

	class CommandBuffer& ktb() const{
		return *(CommandBuffer*)commandBufferHandle;
	}

	void registerSymbol(KrayHandleType type,const char* context=0);
	void freeSymbol();

	void setName(const char* name);
	void generateAutoname();

	void getValue(struct KrayParserExchangeBuffer& kpeb) const;
	void setValue(const struct KrayParserExchangeBuffer& kpeb) const;

	bool validObject() const{
		return getObjectHandle()!=0;
	}
	bool validSymbol() const{
		return getSymbolHandle()!=0;
	}
	void* getObjectHandle() const{
		return directObjectHandle;
	}

protected:
	Symbol(const Symbol& s);

public:
	Symbol(class CommandBuffer& ktb,const char* name=0);
	virtual ~Symbol(){
		freeSymbol();
	}
	void initSymbol(CommandBuffer& ktb,const char* name);

	void registerSymbol();
	bool isRegistered() const;
	
	const char* getName() const{
		return name;
	}
	void* getSymbolHandle() const{
		return directSymbolHandle;
	}
	char* toString(DynamicBuffer& rb);
};

class DatatypeSymbol : public Symbol{
protected:
	void registerSymbol(){
		Symbol::registerSymbol(KrayDtType);
	}

public:
	DatatypeSymbol(CommandBuffer& ktb,const char* name=0):Symbol(ktb,name){
	}
};

class VarLenSymbol : public VarLenTemplate<const Symbol*>{
	const Symbol* s;

protected:
	VarLenSymbol(){
	}

public:
	VarLenSymbol(const Symbol& s0){			// single symbol view
		count=1;
		s=&s0;
		v=&s;
	}
};

class VarLenSymbolDynamic : public VarLenDynamicTemplate<VarLenSymbol>{
public:
	VarLenSymbolDynamic(){
	}
	VarLenSymbolDynamic(size_t size){
		resize(size);
	}
};

};

#endif