/* 
	Kray Vertex map class
*/

#ifndef KRAY_VERTEXMAP_H
#define KRAY_VERTEXMAP_H

#include "../base/vmap.h"
#include "../base/parser.h"
#include "symbol.h"
#include "../buffer/cmd.h"

namespace Kray{

class VertexMapInfo{
	friend class VertexMapSymbol;

	class KrayVertexMap* handle;

	bool doublePrecision;

public:
	VertexMapInfo(){
		clear();
	}

	size_t getSize() const;			// maximum index+1
	int getDimension() const;		// number of vmap columns
	int getSamples() const;			// number of time samples

	template <class T> T* data() const{
		return (T*)handle->data;
	}
	size_t dataIndex(size_t index,int column=0,int sample=0) const{
		return handle->dataIndex(index,column,sample);
	}

	float* getRowFloatPtr(size_t index) const;		// will return 0,if Kray internal format is 'double'
	double* getRowDoublePtr(size_t index) const;	// will return 0,if Kray internal format is 'float'

	// those versions does automatic conversions of format
	float getValueFloat(size_t index,int column,int sample=0) const;		// index - vmap index, shift - columm, sample - time sample
	double getValueDouble(size_t index,int column,int sample=0) const;
	void setValue(size_t index,int column,float value,int sample=0) const;
	void setValue(size_t index,int column,double value,int sample=0) const;
	
	void getRow(size_t index,float* row,int sample=0) const;
	void getRow(size_t index,double* row,int sample=0) const;
	void setRow(size_t index,const float* row,int sample=0) const;
	void setRow(size_t index,const double* row,int sample=0) const;

protected:
	bool ok() const{
		return handle!=0;
	}
	void clear(){
		handle=0;
		doublePrecision=false;
	}
	template <class F> void getRowTemplate(size_t index,F* row,int sample=0) const;
	template <class F> void setRowTemplate(size_t index,const F* row,int sample=0) const;
};

class VertexMapSymbol : public Symbol{
	VertexMapInfo info;

protected:
	void registerSymbol(int dim,size_t size,int animSamples=1);
	void refreshDirectLink();

public:
	VertexMapSymbol(CommandBuffer& ktb,int dim,size_t size=0):Symbol(ktb){	// if size==0, vertexMap is dynamically resized, but this mode does not allow direct access
		registerSymbol(dim,size);
	}
	
	const VertexMapInfo* direct() const;

	void data(size_t index,const Vector& dta);
	void data(size_t index,const VarLenDouble& dta);
};

};

#endif