#ifndef KRAY_COORDSYS_H
#define KRAY_COORDSYS_H

#include "../base/parser.h"
#include "../symbol/symbol.h"
#include "../buffer/cmd.h"

namespace Kray{

class CoordSys : public KrayCoordSysCore{
public:
	CoordSys(){
		clear();
	}
	CoordSys(const Kray::Vector& v,const Kray::Axes& a);
	CoordSys(const KrayCoordSysCore& core);
	CoordSys(const class Matrix4x4& v);

	void clear();
	bool isCenteredAtZero() const;
	bool isEyeAxes() const;

	void set(const KrayVectorCore& vec);
	void set(const KrayAxesCore& ax);

	Axes getAxes() const;
	Vector getVector() const;

	char transform(CoordSys& v,char reverseTransform=0) const;
	char transform(Vector& v,char reverseTransform=0) const;
	
	KrayDtTypes dtType() const{
		return KrayDtVectorType;
	}

	bool isValid() const{
		return a.count>=0;
	}

	void invalidate(){
		a.count=-1;
	}

	int dim() const{
		return krayParserAxesMax;
	}

	void toMatrix(Matrix4x4& result);
};

class CoordSysSymbol : public DatatypeSymbol{
	void registerSymbol();
public:
	CoordSysSymbol(CommandBuffer& ktb,const char* name=0):DatatypeSymbol(ktb,name){
		registerSymbol();
	}	

	void directSet(const CoordSys& v);
	void directGet(CoordSys&) const;
};

class VarLenCoordSys : public VarLenTemplate<CoordSys>{
	VarLenCoordSys(){
	}

public:
	VarLenCoordSys(const CoordSys& v0){	// creates single vector view, NOTE! v0 is referenced, not copied!
		count=1;
		v=(CoordSys*)&v0;
	}
};

template <size_t N> class VarLenCoordSysFixed : public VarLenFixedTemplate<VarLenCoordSys,N>{
};

class VarLenCoordSysDynamic : public VarLenDynamicTemplate<VarLenCoordSys>{
};

};

#endif