#ifndef KRAY_VECTOR_H
#define KRAY_VECTOR_H

#include "../base/parser.h"
#include "../base/varlen.h"
#include "../symbol/symbol.h"
#include "../buffer/cmd.h"

namespace Kray{

class Vector : public KrayVectorCore{
public:
	static double dot(const double* v1,const double* v2,int count);
	static void scale(double* v,int count,double value);
	
public:
	Vector(){
		count=0;
	}
	Vector(double v0,double v1){
		count=2;
		v[0]=v0;
		v[1]=v1;
	}
	Vector(double v0,double v1,double v2){
		count=3;
		v[0]=v0;
		v[1]=v1;
		v[2]=v2;
	}
	Vector(double v0,double v1,double v2,double v3){
		count=4;
		v[0]=v0;
		v[1]=v1;
		v[2]=v2;
		v[3]=v3;
	}
	Vector(const KrayVectorCore& core){
		count=core.count;

		for (int a=0 ; a<count ; a++){
			v[a]=core.v[a];
		}
	}

	static Vector fromArray(const double* vv,int c){
		Vector r;
		r.set(vv,c);

		return r;
	}

	void set(const double* vv,int c){
		count=c;
		for (int a=0 ; a<3 ; a++){
			v[a]=vv[a];
		}
	}
	
	KrayDtTypes dtType() const{
		return KrayDtVectorType;
	}
	
	double operator[](int i) const{
		return v[i];
	}
	double& operator[](int i){
		return v[i];
	}
	
	int dim() const{
		return count;
	}
	double average() const{
		double res=v[0];
		
		for (int a=1 ; a<count ; a++){
			res+=v[a];
		}
		
		return res/count;
	}

	void setXyz(double x,double y,double z){
		count=3;
		v[0]=x;
		v[1]=y;
		v[2]=z;
	}

	void setRgba(double r,double g,double b,double a=1){
		count=4;
		v[0]=r;
		v[1]=g;
		v[2]=b;
		v[3]=a;
	}

	void setRgb(double r,double g,double b){
		count=3;
		v[0]=r;
		v[1]=g;
		v[2]=b;
	}
	void setRgb(double a){
		setRgb(a,a,a);
	}

	void invalidate(){
		count=0;
	}
	bool isValid() const{
		return count!=0;
	}

	double dot(const Vector& v2) const{
		return dot(v,v2.v,count);
	}
	void cross(const Vector& v1,const Vector& v2);
	double length() const;

	Vector& operator*=(double m);
	Vector& operator+=(const Vector& v);
	Vector& operator-=(const Vector& v);

	Vector& copy(const double* vector,int n){
		count=n;

		for (int a=0 ; a<count ; a++){
			v[a]=vector[a];
		}

		return (*this);
	}
	void copyTo(double* vector) const{
		for (int a=0 ; a<count ; a++){
			vector[a]=v[a];
		}
	}
};

class VectorSymbol : public DatatypeSymbol{
	void registerSymbol();
public:
	VectorSymbol(CommandBuffer& ktb,const char* name=0):DatatypeSymbol(ktb,name){
		registerSymbol();
	}	

	void directSet(const Vector& v);
	void directGet(Vector&) const;
};

class VarLenVector : public VarLenTemplate<Vector>{
	VarLenVector(){
	}

public:
	VarLenVector(const Vector& v0){	// creates single vector view, NOTE! v0 is referenced, not copied!
		count=1;
		v=(Vector*)&v0;
	}
};

template <size_t N> class VarLenVectorFixed : public VarLenFixedTemplate<VarLenVector,N>{
};

class VarLenVectorDynamic : public VarLenDynamicTemplate<VarLenVector>{
};

};

#endif