#ifndef ALLOCATE_MEM_H
#define ALLOCATE_MEM_H

#include <stddef.h>

// define this to get rid of stdc++ linkage need
// (you also need to disable RTTI, exceptions and thread safe static
// for GCC or Clang this requires
// -DMEM_USE_STDC_ALLOCATOR -fno-rtti -fno-threadsafe-statics -fno-exceptions)

#ifdef MEM_USE_STDC_ALLOCATOR

#include <stdlib.h>

#define MEM_USE_STDC_MEM_ALLOC malloc
#define MEM_USE_STDC_MEM_FREE free
#endif

#if defined(MEM_USE_STDC_MEM_ALLOC) && defined(MEM_USE_STDC_MEM_FREE)
#define MEM_USE_ALLOCATOR
#endif

class MemoryAllocator{
public:
#ifdef MEM_USE_ALLOCATOR
	template <class T> T* allocateTable(size_t size) const{
		return (T*)MEM_USE_STDC_MEM_ALLOC(size*sizeof(T));
	}
	template <class T> void freeTable(T* memory) const{
		MEM_USE_STDC_MEM_FREE((void*)memory);
	}
	
	void* operator new(size_t size){
		return MEM_USE_STDC_MEM_ALLOC(size);
	}
	void operator delete(void* memory){
		return MEM_USE_STDC_MEM_FREE(memory);
	}
	void* operator new[](size_t size){
		return MEM_USE_STDC_MEM_ALLOC(size);
	}
	void operator delete[](void* memory){
		return MEM_USE_STDC_MEM_FREE(memory);
	}
#else
	template <class T> T* allocateTable(size_t size) const{
		return new T[size];
	}
	template <class T> void freeTable(T* memory) const{
		delete[] memory;
	}
#endif
};

#endif
