#ifndef SDK_KRAY_JNI_H
#define SDK_KRAY_JNI_H

/*
	Kray SDK Java JNI wrapper header
*/

#include <jni.h>

#define DECLAREJNI(x,name,prefix) extern "C" JNIEXPORT x JNICALL Java_##prefix##_##name
#define KRAYJNI(x,name) DECLAREJNI(x,name,com_kraytracing_kray_Library)

#define JNISTRING(string) JniStringConverter(string,env)
#define JAVASTRING(string) (jstring)JniLocalAutoString(string,env)

extern JavaVM* JAVAVMPOINTER;

class JniEnvHolder{
public:
	JniEnvHolder(JNIEnv* env=0){
	}

	static JNIEnv* getJniEnv(){
		JNIEnv* r;

		if (JAVAVMPOINTER->GetEnv((void**)&r, JNI_VERSION_1_6)!=JNI_OK){
			return 0;
		}

		return r;
	}
	JNIEnv* getEnv() const{
		return getJniEnv();
	}
};

class JniStringConverter : public JniEnvHolder{
	jstring js;
	const char* cs;

public:
	JniStringConverter(jstring string,JNIEnv* env=0):JniEnvHolder(env),js(string){
		cs=getEnv()->GetStringUTFChars(js,0);
	}
	~JniStringConverter(){
		getEnv()->ReleaseStringUTFChars(js,cs);
	}

	operator const char*() const{
		return cs;
	}
};

class JniLocalAutoObject : public JniEnvHolder{
protected:
	jobject reference;

protected:
	JniLocalAutoObject(JNIEnv* env):JniEnvHolder(env),reference(0){
	}

public:
	JniLocalAutoObject(jobject j=0,JNIEnv* env=0):JniEnvHolder(env),reference(0){
		set(j);
	}
	~JniLocalAutoObject(){
		clear();
	}

	void set(jobject j){
		clear();
		if (j){
			reference=getEnv()->NewLocalRef(j);
		}
	}
	void clear(){
		if (reference){
			getEnv()->DeleteLocalRef(reference);
			reference=0;
		}		
	}

	operator jobject() const{
		return (jobject)reference;
	}
};

class JniLocalAutoString : public JniEnvHolder{
public:
	jstring string;

public:
	JniLocalAutoString(const char* txt,JNIEnv* env=0):JniEnvHolder(env){
		string=getEnv()->NewStringUTF(txt);
	}
	~JniLocalAutoString(){
		if (string){
			getEnv()->DeleteLocalRef(string);
		}
	}

	operator jstring() const{
		return string;
	}
};

class JniGlobalReference : public JniLocalAutoObject{
public:
	JniGlobalReference(jobject obj=0,JNIEnv* env=0):JniLocalAutoObject(env){
		set(obj);
	}
	~JniGlobalReference(){
		clear();
	}

	void set(jobject j){
		clear();
		if (j){
			reference=getEnv()->NewGlobalRef(j);
		}
	}
	void clear(){
		if (reference){
			getEnv()->DeleteGlobalRef(reference);
			reference=0;
		}
	}
};

#endif
