#include "resizablebuffer.h"
#include "str2num.h"

void ResizableBuffer::align(size_t alignment){
	size_t adress=(size_t)buffer+used;
	
	size_t misalignment=(adress+alignment-1)/alignment;
	misalignment*=alignment;
	misalignment-=adress;	// we are aligning to absolute adress, not 'used' offset
	
	if (misalignment){
		allocBuffer(misalignment);
	}
}
size_t ResizableBuffer::allocBuffer(size_t size){
	fit(used+size);
	
	size_t result=used;
	
	used+=size;
	
	return result;
}
void ResizableBuffer::freeBuffer(){
	freeTable(buffer);

	buffer=0;
	allocated=0;
	used=0;
}
void ResizableBuffer::replace(size_t dest,const char* src,size_t size){
	if (buffer){
		s2n_memcpy(buffer+dest,src,size);
	}
}
char ResizableBuffer::append(const char* data,size_t size){	
	size_t dest=allocBuffer(size);

	if (!getBuffer()) return 0;

	replace(dest,data,size);

	return 1;
}
char ResizableBuffer::appendString(const char* str){
	return append(str,s2n_strlen(str));
}
char ResizableBuffer::append(char data){
	size_t i=allocBuffer(1);

	if (buffer){
		buffer[i]=data;

		return 1;
	}

	return 0;
}
void ResizableBuffer::resize(size_t new_alloc){
	char* new_buffer=allocateTable<char>(new_alloc);

	if (buffer){
		s2n_memcpy(new_buffer,buffer,used+1);

		freeTable(buffer);
	}

	buffer=new_buffer;
	allocated=buffer?new_alloc:0;
}
void ResizableBuffer::fit(size_t new_size){
	if (new_size>allocated){
		size_t new_alloc=allocated;

		if (!new_alloc){
			new_alloc=16;
		}

		while(new_alloc<new_size){
			new_alloc*=2;
		}

		resize(new_alloc);
	}
}
