#ifndef RESIZABLE_BUFFER_H
#define RESIZABLE_BUFFER_H

#include <stddef.h>

#include "allocator.h"

class ResizableBufferBase : public MemoryAllocator{
protected:
	char* buffer;

	size_t allocated;

protected:
	ResizableBufferBase():buffer(0),allocated(0){
	}

public:
	char* getBuffer() const{
		return buffer;
	}
	size_t size() const{
		return allocated;
	}
};

class ResizableBuffer : public ResizableBufferBase{
protected:
	size_t used;	// without terminating 0

public:
	ResizableBuffer(){
		freeBuffer();
	}
	~ResizableBuffer(){
		freeBuffer();
	}

	void resize(size_t new_alloc);
	void fit(size_t new_size);
	void align(size_t alignment);
	size_t allocBuffer(size_t size);

	void replace(size_t dest,const char* src,size_t size);
	char append(const char* data,size_t size);
	char append(char data);
	char appendString(const char* str);
	void freeBuffer();
	
	void truncate(){
		used=0;
	}
	size_t cursor() const{
		return used;
	}

	template <class T> void replaceBinary(size_t offset,const T& v){
		replace(offset,(const char*)&v,sizeof(v));
	}
	template <class T> void appendBinary(const T& v){		// returns offset of added data
		align(sizeof(v));

		append((const char*)&v,sizeof(v));
	}
};

#endif