#include "mathcpp.h"
#include "str2num.h"

template <class T> inline T absT(T a){
	if (a<0){
		return -a;
	}

	return a;
}
inline char _is_digit(char a){
	return (a>='0' && a<='9');
}
inline char _hex_value(char a){
	if (_is_digit(a)){
		return a-'0';
	}else if (a>='a' && a<='z'){
		return a-'a'+10;
	}else if (a>='A' && a<='Z'){
		return a-'A'+10;
	}else{
		return 127;
	}
}
inline char _hex_char(char a){
	if (a<10){
		return a+'0';
	}else{
		return a-10+'A';
	}
}
inline char _hex_char_case(unsigned char a){
	const unsigned char limit=(10+'Z'-'A'+1);

	if (a<limit){
		return _hex_char(a);
	}else{
		return a-limit+'a';
	}
}
inline char _is_digit(char a,int modulo){
	return _hex_value(a)<modulo;
}
char _ddigit(double n,int digit){
	double m=pow(10,(double)(-digit+1));
	
	n=floor(n*m);
	m=floor(n/10);
	n-=m*10;
	
	char res=(char)(n+0.5);

	if (res<0) res=0;
	if (res>9) res=9;
	
	return res;
}

size_t modulo_2_size_t(const char*& str,int modulo){
	size_t result=0;
	
	if (_is_digit(*str,modulo)){
		for (;;){
			result+=_hex_value(*str);
			str++;
			if (!_is_digit(*str,modulo)) break;
			result*=modulo;
		}
	}
	
	return result;
}
double udecimal_2_double(const char*& str){
	double result=0;

	if (_is_digit(*str)){
		for (;;){
			result+=*str-'0';
			str++;
			if (!_is_digit(*str)) break;
			result*=10;
		}
	}
	
	if (*str=='.'){
		size_t fraction=10;
		
		str++;

		if (_is_digit(*str)){
			for (;;){
				result+=(double)(*str-'0')/fraction;
				str++;
				if (!_is_digit(*str)) break;
				fraction*=10;
			}
		}		
	}

	if (*str=='e' || *str=='E'){
		str++;
		
		if (*str=='+'){
			str++;
		}
		
		int sexp=modulo_2_int(str);
		
		result*=pow(10.0,(double)sexp);
	}
	
	return result;
}
double sdecimal_2_double(const char* &str){
	double result=1;
	
	if (*str=='-'){
		result=-1;
		str++;
	}
	
	return result*udecimal_2_double(str);
}
int modulo_2_int(const char*& str,int modulo){
	int result=1;
	
	if (*str=='-'){
		result=-1;
		str++;
	}
	
	return result*(int)modulo_2_size_t(str,modulo);
}


////////////////////////////////////////////////////////

void fill_and_minus(char*& buffer,size_t& bufsize,size_t flen,char fill,char minus){	// flen is full length
	if (fill && bufsize>flen){
		flen=bufsize-flen;		// flen is fill length now

		if (minus && flen>0) flen--;

		for (size_t a=0 ; a<flen ; a++){
			*buffer++=fill;
		}
			
		bufsize-=flen;
	}

	if (minus && bufsize>0){
		*buffer++='-';
		bufsize--;
	}
}

void size_t_2_modulo(size_t n,char* buffer,size_t& bufsize,char fill,int modulo,char minus){
	if (!bufsize) return;
	
	bufsize--;	// string limiter
	
	size_t limit=1;
	size_t olimit;
	size_t count=0;
	
	do{
		olimit=limit;
		limit*=modulo;
		count++;
	}while((limit/modulo)==olimit && limit<=n);

	fill_and_minus(buffer,bufsize,count,fill,minus);
	
	while(bufsize>0 && olimit>0){
		char digit=(char)(n/olimit);
		*buffer++=_hex_char_case(digit);
		n-=digit*olimit;
		olimit/=modulo;
		bufsize--;
	}
	
	*buffer=0;	// string limiter
}


void int_2_modulo(int n,char* buffer,size_t& bufsize,char fill,int modulo){
	if (n<0){
		if (bufsize>1){
			size_t_2_modulo((size_t)-n,buffer,bufsize,fill,modulo,1);
		}else{
			*buffer=0;
			bufsize--;
		}
	}else{
		size_t_2_modulo((size_t)n,buffer,bufsize,fill,modulo,0);
	}
}

int double_predot_count(double n){
	int predot=1;

	if (n!=0.0){
		predot=(int)(log(n)/log(10.0))+1;

		if (_ddigit(n,predot+1)!=0){
			predot++;
		}
		
		if (predot<=0) predot=1;
	}

	return predot;
}

void double_2_decimal(double n,char* buffer,size_t& bufsize,int dot,char fill){
	if (!bufsize) return;

	bufsize--;	// terminator

	char minus=0;
	if (n<0){
		minus=1;
		n*=-1;
	}
	
	int predot=double_predot_count(n);	// ilosc cyfr przed przecinkiem
	
	{
		size_t flen=dot+predot;

		if (dot) flen++;

		fill_and_minus(buffer,bufsize,flen,fill,minus);
	}
	
	while(predot>0 && bufsize>0){
		*buffer++=_ddigit(n,predot)+'0';
		predot--;
		bufsize--;
	}
	if (bufsize && dot){
		*buffer++='.';
		bufsize--;

		while(bufsize && dot){
			*buffer++=_ddigit(n,predot)+'0';
			predot--;
			bufsize--;
			dot--;
		}
	}
	
	*buffer=0;
}

void double_2_sdecimal(double n,char* buffer,size_t& bufsize,int dot,char fill){
	if (bufsize>1){
		double_2_decimal(n,buffer,bufsize,dot,fill);
	}else{
		*buffer=0;
		bufsize--;
	}
}

static int _dots(double n,int dot){
	for (int d=dot ; d>0 ; d--){
		if (_ddigit(n,(1-d))!=0){
			return d;
		}
	}
	
	return 0;
}
void double_2_natural(double n1,char* buffer,size_t& bufsize,int dot,char fill){	// clips trailing zeros
	char sign=1;
	
	if (n1<0){
		sign=-1;
		n1*=-1;
	}
	
	double n2=n1+5*pow(10.0,-(double)(dot+1));	// last digit rounding

	int d1=_dots(n1,dot);
	int d2=_dots(n2,dot);
	
	if (d1<=d2){
		double_2_sdecimal(n1*sign,buffer,bufsize,d1,fill);
	}else{
		double_2_sdecimal(n2*sign,buffer,bufsize,d2,fill);		
	}
}

void double_2_sci(double n,char* buffer,size_t& bufsize,int dot,int sci_tolerance){
	double exponens=log(absT(n))/log(10.0);

	int iexp=(int)floor(exponens);
	
	char sci=0;
	
	if (absT(iexp)>sci_tolerance){
		sci=1;
	}
	
	if (sci){
		n/=pow(10.0,(double)iexp);
	}
	
	size_t obsize=bufsize;
	
	double_2_natural(n,buffer,bufsize,dot);

	if (bufsize>1 && sci){
		bufsize++;	// erase terminating zero
		buffer+=obsize-bufsize;

		*buffer++='e';
		bufsize--;
		
		int_2_modulo(iexp,buffer,bufsize);
	}
}

/////////////////////////////////////////////////////////////////////////////////////

size_t s2n_strlen(const char* string){
	size_t r=-1;

	while(string[++r]);

	return r;
}
void s2n_strncpy(char* dest,const char* src,size_t max){
	do{
		if (!max--) break;

		*dest++=*src;
	}while(*src++);
}
void s2n_strcpy(char* dest,const char* src){
	do{
		*dest++=*src;
	}while(*src++);
}
void s2n_strcat(char* dest,const char* src){
	s2n_strcpy(dest+s2n_strlen(dest),src);
}
int s2n_strcmp(const char* s1,const char* s2){
	for(;;){
		if (*s1<*s2) return -1;
		if (*s1>*s2) return 1;

		if (!*s1) return 0;

		s1++;
		s2++;
	}
}
int s2n_strncmp(const char* s1,const char* s2,size_t max){
	for(; max>0 ; max--){
		if (*s1<*s2) return -1;
		if (*s1>*s2) return 1;

		if (!*s1) return 0;

		s1++;
		s2++;
	}

	return 0;
}
const char* s2n_strstr(const char* text,const char* find){
	do{
		size_t si=0;

		while(text[si]==find[si] && find[si]!=0){
			si++;
		}

		if (find[si]==0) return text;
	}while(*text++);

	return 0;
}
char* s2n_strstr(char* text,const char* find){
	return (char*)s2n_strstr((const char*)text,find);
}

/////////////////////////////////////////////////////////////////////////////////////

template <class T> inline void s2n_tabcpy(T* dest,const T* src,size_t size){
	for (size_t i=0 ; i<size ; i++){
		dest[i]=src[i];
	}
}
void s2n_amemset(void* dest,unsigned char value,size_t size){
	size_t v=(unsigned char)value;

	for (int b=1 ; b<sizeof(size_t) ; b*=2){
		v|=v<<(8*b);
	}

	size/=sizeof(size_t);

	for (size_t a=0 ; a<size ; a++){
		((size_t*)dest)[a]=v;
	}
}
int s2n_memcmp(void* _s1,void* _s2,size_t n){
	unsigned char* s1=(unsigned char*)_s1;
	unsigned char* s2=(unsigned char*)_s2;

    for ( ; n-- ; s1++, s2++){
		int c=(*s1-*s2);

		if (c){
			return c;
		}
    }

    return 0;
}
void s2n_memset(void* dest,unsigned char value,size_t size){
	if ( ((size_t)dest | size) & (sizeof(size_t)-1) ){		// misaligned?
		for (size_t a=0 ; a<size ; a++){
			((unsigned char*)dest)[a]=value;
		}
	}else{
		s2n_amemset(dest,value,size);
	}
}

void s2n_amemcpy(void* dest,const void* src,size_t size){
	s2n_tabcpy((size_t*)dest,(const size_t*)src,size/sizeof(size_t));
}
void s2n_memcpy(void* dest,const void* src,size_t size){
	if ( ((size_t)dest | (size_t)src | size) & (sizeof(size_t)-1)){		// misaligned?
		s2n_tabcpy((char*)dest,(const char*)src,size);
	}else{
		s2n_amemcpy(dest,src,size);
	}
}
void s2n_numcat(void* n,char* buffer){
	buffer+=s2n_strlen(buffer);
	
	size_t buflen=30;
	
	size_t_2_modulo((size_t)n,buffer,buflen,0,16);
}
void s2n_numcat(int n,char* buffer){
	buffer+=s2n_strlen(buffer);

	size_t buflen=30;
	
	int_2_modulo(n,buffer,buflen);
}

void s2n_numcat(double n,char* buffer,size_t blen,int dot){
	buffer+=s2n_strlen(buffer);

	size_t buflen=blen;

	double_2_sdecimal(n,buffer,buflen,dot);
}
void s2n_numcat(float n,char* buffer){
	buffer+=s2n_strlen(buffer);

	size_t buflen=12;

	double_2_sdecimal(n,buffer,buflen,5);
}

void s2n_numcat(unsigned int n,char* buffer,int width){
	buffer+=s2n_strlen(buffer);

	size_t buflen=width+1;
	
	size_t_2_modulo(n,buffer,buflen,'0');
}

void s2n_numsrd(const char* prefix,int n,const char* suffix,char* buffer){
	if (prefix){
		s2n_strcpy(buffer,prefix);
	}else{
		*buffer=0;
	}
	
	s2n_numcat(n,buffer);
	
	if (suffix){
		s2n_strcat(buffer,suffix);
	}
}
