#ifndef UTF8_CODEC_HEADER_ef5e765e7f5_H
#define UTF8_CODEC_HEADER_ef5e765e7f5_H

#include <stddef.h>

#include "memhneeded.h"

class UTF8Codec{
	static char checkHead(int& head,int mask,int value){
		if ((head&(~mask))==value){
			head&=mask;
			
			return 1;
		}
		
		return 0;
	}
	static char getNext(const unsigned char*& dest,int& next){
		next=(*++dest);
		if (!next) return 0;
		
		if ((next&0xc0)!=0x80){
			return 0;
		}
		
		next&=~0xc0;
		
		return 1;
	}
	static int decodeUnsafe(const char* _src,unsigned int &c){
		const unsigned char *src=(const unsigned char*)_src;

		int byte=(*src);
		
		if (byte&0x80){
			if (checkHead(byte,0x1f,0xc0)){
				c=byte<<(11-5);
				
				if (!getNext(src,byte)) return 0;
				c|=byte;
				
				return 2;
			}
			
			if (checkHead(byte,0xf,0xe0)){
				c=byte<<(16-4);
				
				if (!getNext(src,byte)) return 0;
				c|=byte<<(16-10);

				if (!getNext(src,byte)) return 0;
				c|=byte;

				return 3;
			}
			
			if (checkHead(byte,0x7,0xf0)){
				c=byte<<(21-3);
				
				if (!getNext(src,byte)) return 0;
				c|=byte<<(21-9);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(21-15);

				if (!getNext(src,byte)) return 0;
				c|=byte;

				return 4;
			}
			
			if (checkHead(byte,0x3,0xf8)){
				c=byte<<(26-2);
				
				if (!getNext(src,byte)) return 0;
				c|=byte<<(26-8);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(26-14);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(26-20);

				if (!getNext(src,byte)) return 0;
				c|=byte;

				return 5;
			}
			
			if (checkHead(byte,0x1,0xfc)){
				c=byte<<(31-1);
				
				if (!getNext(src,byte)) return 0;
				c|=byte<<(31-7);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(31-13);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(31-19);

				if (!getNext(src,byte)) return 0;
				c|=byte<<(31-26);

				if (!getNext(src,byte)) return 0;
				c|=byte;

				return 6;
			}
			
			return -1;	// invalid sequence head
		}else{
			c=byte;

			return 1;
		}
	}
public:
	static char skipBOM(const char*& c){	// skips byte order mark (often on the begining of utf8 encoded file to mark its format)
		if (c[0]!=(char)0xef) return 0;
		if (c[1]!=(char)0xbb) return 0;
		if (c[2]!=(char)0xbf) return 0;
		
		c+=3;
		
		return 1;
	}
	static const char* BOM(){
		static unsigned char _bom[]={0xef,0xbb,0xbf,0};
		return (char*)_bom;
	}
	static int length(unsigned int c){
		if (c<0x80) return 1;
		if (c<0x800) return 2;
		if (c<0x10000) return 3;
		if (c<0x200000) return 4;
		if (c<0x4000000) return 5;
		if (c<0x80000000) return 6;

		return -1;
	}
	static int length(const char* src){
		unsigned int dummy;
		
		return decode(src,dummy);
	}
	static void encode(unsigned int c,char *dest){
		if (c<0x80){
			*dest++=(char)c;
		}else if (c<0x800){
			*dest++=(c>>(11-5))			| 0xc0;
			*dest++=((c)&0x3f)			| 0x80;
		}else if (c<0x10000){
			*dest++=(c>>(16-4))			| 0xe0;
			*dest++=((c>>(16-10))&0x3f)	| 0x80;
			*dest++=((c)&0x3f)			| 0x80;
		}else if (c<0x200000){
			*dest++=(c>>(21-3))			| 0xf0;
			*dest++=((c>>(21-9))&0x3f)	| 0x80;
			*dest++=((c>>(21-15))&0x3f)	| 0x80;
			*dest++=((c)&0x3f)			| 0x80;
		}else if (c<0x4000000){
			*dest++=(c>>(26-2))			| 0xf8;
			*dest++=((c>>(26-8))&0x3f)	| 0x80;
			*dest++=((c>>(26-14))&0x3f)	| 0x80;
			*dest++=((c>>(26-20))&0x3f)	| 0x80;
			*dest++=((c)&0x3f)			| 0x80;
		}else{
			*dest++=(c>>(31-1))			| 0xfc;
			*dest++=((c>>(31-7))&0x3f)	| 0x80;
			*dest++=((c>>(31-13))&0x3f)	| 0x80;
			*dest++=((c>>(31-19))&0x3f)	| 0x80;
			*dest++=((c>>(31-25))&0x3f)	| 0x80;
			*dest++=((c)&0x3f)			| 0x80;
		}		
	}
	static int decode(const char* src,unsigned int& c){
		int len=decodeUnsafe(src,c);
		
		if (len!=length(c)){
			return -1;	// not minimal sequence
		}
		
		return len;
	}
};

class UTF8Tools{
public:
	static int minimalIntSize(unsigned int c){	// returns minimum size required to strore int
		if (c<0x100) return 1;
		if (c<0x10000) return 2;
		
		return 4;
	}
	static size_t stringUnicodeLength(const char *utf){	// returns number of unicode decoded chars, returns -1 (unsigned) if string is invalid
		size_t unicodechars=0;
			
		while(utf[unicodechars]!=0){
			int len=UTF8Codec::length(utf+unicodechars);
			
			if (len<0){
				return -1;
			}
			
			unicodechars+=len;
		}
		
		return unicodechars;
	}
};

template <class T> class UTF8Buffer{
protected:
	T* data;
	size_t dataSize;

public:
	UTF8Buffer():data(0),dataSize(0){
	};
	~UTF8Buffer(){
		free();
	}
	
	void free(){
		delete[] data;
		data=0;
		dataSize=0;
	}
	char alloc(size_t size){	// alloc given number of bytes (with termination zero)
		dataSize=size;
		data=new T[dataSize];
		if (!data) return 0;
		
		data[size-1]=0;
		
		return 1;
	}
	char fit(size_t size){	// fit string with given length
		size++;
		
		if (dataSize<size){
			return alloc(size);
		}
		
		return 1;
	}

	const T* buffer() const{
		return data;
	}
	T* buffer(){
		return data;
	}
	
	operator T*(){
		return buffer();
	}
	operator const T*() const{
		return buffer();
	}
	
	T& operator[](int a){
		return data[a];
	}
	const T& operator[](int a) const{
		return data[a];
	}

	static size_t length(const T* data){
		size_t ret=0;
		
		while(data[ret]!=0){
			ret++;
		}
		
		return ret;
	}
	size_t length() const{
		return length(data);
	}
};

template <class T> class UTF8DecoderBuffer : public UTF8Buffer<T>{
public:
	UTF8DecoderBuffer(){
	}
	UTF8DecoderBuffer(const char* src,char allowClip=0){
		if (!decode(src,allowClip)){
			UTF8Buffer<T>::free();
		}
	}

	char decode(const char* src,T clipChar=0){	// return 0 if failed
		size_t len=UTF8Tools::stringUnicodeLength(src);
		
		if (len==-1) return 0;	// invalid string
		
		UTF8Buffer<T>::fit(len);

		return decode(src,UTF8Buffer<T>::data,clipChar);
	}
	static char decode(const char* src,T* dest,T clipChar=0){
		size_t len=0;
		
		while(*src){
			unsigned int c;
			int l=UTF8Codec::decode(src,c);
			
			if (l<0) return 0;
			
			if (UTF8Tools::minimalIntSize(c)>sizeof(T)){
				if (clipChar==0){
					return 0;
				}else{
					c=clipChar;
				}
			}
			
			dest[len++]=(T)c;
			src+=l;
		}
		
		dest[len]=0;
		
		return 1;
	}
};

template <class T> class UTF8EncoderBuffer : public UTF8Buffer<char>{
public:
	UTF8EncoderBuffer(){
	}
	UTF8EncoderBuffer(const T* src){
		if (!encode(src)){
			free();
		}
	}

	char encode(const T* src){
		size_t len=stringUTF8Length(src);
		
		if (len==-1) return 0;
		if (!UTF8Buffer<char>::fit(len)) return 0;

		return encode(src,data);
	}	

	static size_t stringUTF8Length(const T* src){	// returns number of characters for utf8 string
		size_t ret=0;
		
		while((*src)!=0){
			int l=UTF8Codec::length(*src);			
			if (l<0) return -1;
			
			ret+=l;
			src++;
		}
		
		return ret;	
	}
	static char encode(const T* src,char *dest){
		while((*src)!=0){
			UTF8Codec::encode(*src,dest);
			
			dest+=UTF8Codec::length(*src);
			src++;
		}
		
		*dest=0;

		return 1;
	}
};

#endif
