#include "../../kraysdk/kray.h"
#include "../../kraysdk/staticlink.h"
#include "../../kraysdk/wrappers/c.h"
#include "../../kraysdk/proto/base.h"
#include "../../kraysdk/proto/direct.h"

using namespace Kray;

// info
KRAYC(const char*) KrayGetInfoString(int mode){
	return getLibrary().getInfoString(mode);	
}
KRAYC(int) KrayGetInfoInt(int mode){
	return getLibrary().getInfoInt(mode);
}
KRAYC(int) KrayGetHostVersion(){
	return getLibrary().getHostApiVersion();
}
KRAYC(int) KrayGetLibApiVersion(){
	return getLibrary().getLibApiVersion();
}

// create functions will load library if needed, but such automation is not thread safe
// for thread safe create/release load libray first with LibraryLoad
KRAYC(KrayInstanceHandle) KrayLibraryCreateInstanceWithEventHandler(KrayEventHandler handler,void* userData){
	return getLibrary().createInstanceWithHandler((EventHandler)handler,userData);
}
KRAYC(KrayInstanceHandle) KrayLibraryCreateInstance(){
	return getLibrary().createInstance();
}
KRAYC(void) KrayLibraryReleaseInstance(KrayInstanceHandle instance){
	getLibrary().releaseInstance((Instance*)instance);
}

KRAYC(int) KrayLibraryLoad(const char* libraryDir){		// explicit call required when one wants to use thread safe create/release instance
	getLibrary().clrFlag(Library::flagUnloadIfEmpty);

	return getLibrary().load(libraryDir);
}
KRAYC(void) KrayLibraryUnload(){		// must be called only when LibraryLoad was called explicitly
	getLibrary().setFlag(Library::flagUnloadIfEmpty);

	getLibrary().release();
}
KRAYC(int) KrayInstanceParse(KrayInstanceHandle instance,const char* krayScript,const char* optionalKrayScriptName){
	return ((Instance*)instance)->parse(krayScript,optionalKrayScriptName);
}

/////////////////////////////////////////////////////////////////

KRAYC(void) KrayInstanceSendBackEvent(KrayEventData* eventData){
	((Instance*)(eventData->instance))->sendBackEvent(*(EventData*)eventData);
}
KRAYC(KrayHandle) KrayInstanceGetObjectByName(KrayInstanceHandle instance,const char* name,KrayHandleType type){
	return ((Instance*)instance)->getObjectByName(name,type);
}
KRAYC(void) KrayInstanceGetSymbolValue(KrayInstanceHandle instance,KrayDtHandle symbolHandle,KrayParserExchangeBuffer* data){
	Instance* i=(Instance*)instance;

	i->vtableParser()->getValue(i->getDefaultParser(),symbolHandle,data);
}
KRAYC(const char*) KrayInstanceGetStringSymbolValue(KrayInstanceHandle instance,KrayDtHandle symbolHandle){
	KrayParserExchangeBuffer data;
	data.dtType=KrayDtStringType;

	KrayInstanceGetSymbolValue(instance,symbolHandle,&data);

	return data.u.string.string;
}

KRAYC(KrayProtoHandle) KrayCreateDirectProto(KrayInstanceHandle instance){
	if (!instance) return 0;

	return new DirectPrototyper((Instance*)instance);
}
KRAYC(void) KrayReleaseProto(KrayProtoHandle handle){
	delete (ProtoBase*)handle;
}
