#ifndef SDK_KRAY_C_H
#define SDK_KRAY_C_H

/*
	Kray SDK ANSI C wrapper header
*/

#include "../base/commons.h"
#include "../base/calling.h"
#include "../base/prscall.h"
#include "../base/event.h"
#include "../proto/base.h"

#ifndef KRAYC
# ifdef KRAY_C_EMSCRIPTEN
#  include <emscripten.h>
#  define KRAYC(x) EMSCRIPTEN_KEEPALIVE x
# elif defined(KRAY_C_DLL)
#  include "../../kraysdk/utils/dl.h"
#  define KRAYC(x) DLLEXPORTFUNCTIONPREFIX x KRAYDLL_CALL_TYPE
# endif
#endif

#ifndef KRAYC
#define KRAYC(x) x
#endif

#ifndef KRAYCEXTERN
#define KRAYCEXTERN(x) KRAYC(x)
#endif

#ifdef __cplusplus
extern "C"{
#endif

typedef void (KRAYDLL_CALL_TYPE *KrayEventHandler)(KrayEventData*);

// Kray library
KRAYCEXTERN(int) KrayLibraryLoad(const char* libraryDir);
KRAYCEXTERN(KrayInstanceHandle) KrayLibraryCreateInstance();
KRAYCEXTERN(KrayInstanceHandle) KrayLibraryCreateInstanceWithEventHandler(KrayEventHandler handler,void* userData);

// Kray instance
KRAYCEXTERN(void) KrayInstanceSendBackEvent(KrayEventData* eventData);
KRAYCEXTERN(void) KrayLibraryReleaseInstance(KrayInstanceHandle instance);

// Kray instance parse calls
KRAYCEXTERN(int) KrayInstanceParse(KrayInstanceHandle instance,const char* krayScript,const char* optionalKrayScriptName);	// last param can be 0

// Kray object access
KRAYCEXTERN(KrayHandle) KrayInstanceGetObjectByName(KrayInstanceHandle instance,const char* name,KrayHandleType type);
KRAYCEXTERN(void) KrayInstanceGetSymbolValue(KrayInstanceHandle instance,KrayDtHandle symbolHandle,struct KrayParserExchangeBuffer* data);
KRAYCEXTERN(const char*) KrayInstanceGetStringSymbolValue(KrayInstanceHandle instance,KrayDtHandle symbolHandle);

// Prototyper
KRAYCEXTERN(KrayProtoHandle) KrayCreateDirectProto(KrayInstanceHandle instance);
KRAYCEXTERN(void) KrayReleaseProto(KrayProtoHandle handle);

#ifdef __cplusplus
}
#endif

#undef KRAYCEXTERN

#endif
