  unit  kray3event;


interface


  uses
    kray.commons;



  // text flags
  const
    KRAY_TEXT_GET_INTERRUPT = 1;
    KRAY_TEXT_GET_UNICODE   = 2;



  // window flags
  const
    KRAY_WINDOW_GET_INTERRUPT     = 1;
    KRAY_WINDOW_GET_FOCUS         = 2;
    KRAY_WINDOW_SET_MOUSE_CAPTURE = 4;



  // window resize flags
  const
    KRAY_WINDOW_ALPHA     = 1;
    KRAY_WINDOW_RESIZABLE = 2;
    KRAY_WINDOW_NOCLOSE   = 4;
    KRAY_WINDOW_NOBORDERS = 8;



  const
    KRAY_MOUSE_CAPTURE_DPI = 1000;



  type
    TKrayEventType = (
                        // draw
                        krayEventSetPixelArray,
                        krayEventSetPixelRectangle,

                        // text stream (log)
                        krayEventCreateTextStream,
                        krayEventDestroyTextStream,
                        krayEventText,
                        krayEventNewLine,
                        krayEventGetText,
                        krayEventGetTextProperty,

                        // window (preview)
                        krayEventCreateWindow,
                        krayEventDestroyWindow,

                        krayEventResizeWindow,
                        krayEventSetWindowTitle,

                        krayEventSetWindowProperty,
                        krayEventGetWindowProperty,
                        krayEventGetWindowMousePosition,
                        krayEventGetWindowMouseRawPosition,
                        krayEventGetWindowKeyEvent,

                        // common
                        krayEventAllocTextBuffer,   // allocates text buffer (optional, called from getTextLine() when textBufferSize==0)
                        krayEventFreeTextBuffer,

                        // terminator
                        krayEventLast
                     );



  type
    TKrayWindowProperties = (krayWindowPropertyFlags, krayWindowPropertyRawMouseDPI);



  type
    TKrayTextProperties = (krayTextPropertyFlags);



  type
    PKrayEventPixel = ^TKrayEventPixel;
    TKrayEventPixel = record
                        x: int;
                        y: int;
                        c: int;  // todo: TColor???
                      end;



  type
    PKrayEventPixelTable = ^TKrayEventPixelTable;
    TKrayEventPixelTable = record
                             pxls : PKrayEventPixel;
                             count: size_t;
                           end;



  type
    PKrayEventPixelRectangle = ^TKrayEventPixelRectangle;
    TKrayEventPixelRectangle = record
                                 pixels: ^uint;
                                 modulo: int;
                                 x1    : int;
                                 y1    : int;
                                 x2    : int;
                                 y2    : int;
                               end;



  type
    PKrayEventWindow = ^TKrayEventPixelRectangle;
    TKrayEventWindow = record
                         positionX: int;
                         positionY: int;
                         width    : int;
                         height   : int;
                         flags    : int;
                       end;



  type
    PKrayEventKey = ^TKrayEventKey;
    TKrayEventKey = record
                      keyCode    : int;
                      miliSeconds: uint;
                      _type      : ansichar;  // char ?
                      pressed    : ansichar;  // char ?
                    end;



  type
    PKrayEventMousePosition = ^TKrayEventMousePosition;
    TKrayEventMousePosition = record
                                x: int;
                                y: int;
                              end;



  type
    PKrayEventTextBufferData = ^TKrayEventTextBufferData;
    TKrayEventTextBufferData = record
                                 buffer           : pansichar;   // pointer to chars
                                 bufferSize       : size_t;      // size of chars space
                                 bufferControlData: pointer;     // user struct for buffer allocation (optional if buffer is not constant size)
                                                                 // non zero value of this field indicates, that buffer needs to be freed with
                                                                 // KrayFreeTextBuffer event
                               end;



  type
    TKrayEventData1 = record
                        case  integer  of
                          0: (windowHandle    : pointer);
                          1: (textStreamHandle: pointer);
                      end;



  type
    TKrayEventData2 = record
                        case  integer  of
                          0: (bitmap        : TKrayBitmapHandle);
                          1: (pixelTable    : PKrayEventPixelTable);
                          2: (pixelRectangle: PKrayEventPixelRectangle);
                          3: (window        : PKrayEventWindow);
                          4: (mousePosition : PKrayEventMousePosition);
                          5: (key           : PKrayEventKey);
                          6: (textBuffer    : PKrayEventTextBufferData);  // text output, const size or resizable buffer for chars (getTextLine)
                          7: (propertyValue : size_t);                    // interupted, unicode support of text input/output
                       end;



  type
    TKrayEventData3 = record
                        case  integer  of
                          0: (text        : pansichar);   // text input
                          1: (propertyType: int);    // rozmiar pointera
                      end;



// hray_loader.cpp

  type
    TKrayEventSetPixelArray             = record
                                            windowHandle    : pointer;
                                            pixelTable      : PKrayEventPixelTable;
                                            _fill_3         : pointer;
                                          end;


    TKrayEventSetPixelRectangle         = record
                                            windowHandle    : pointer;
                                            pixelRectangle  : PKrayEventPixelRectangle;
                                            _fill_3         : pointer;
                                          end;



    TKrayEventCreateWindow              = record
                                            windowHandle    : pointer;
                                            _fill_2         : pointer;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventDestroyWindow             = record
                                            windowHandle    : pointer;
                                            _fill_2         : pointer;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventGetWindowProperty         = record
                                            windowHandle    : pointer;
                                            propertyValue   : size_t;
                                            propertyType    : int;
                                          end;
    TKrayEventSetWindowProperty         = record
                                            windowHandle    : pointer;
                                            propertyValue   : size_t;
                                            propertyType    : int;
                                          end;
    TKrayEventResizeWindow              = record
                                            windowHandle    : pointer;
                                            window          : PKrayEventWindow;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventSetWindowTitle            = record
                                            windowHandle    : pointer;
                                            _fill_2         : pointer;
                                            text            : pansichar;
                                          end;
    TKrayEventGetWindowMousePosition    = record
                                            windowHandle    : pointer;
                                            mousePosition   : PKrayEventMousePosition;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventGetWindowMouseRawPosition = record
                                            windowHandle    : pointer;
                                            mousePosition   : PKrayEventMousePosition;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventGetWindowKeyEvent         = record
                                            windowHandle    : pointer;
                                            key             : PKrayEventKey;
                                            _fill_3         : pointer;
                                          end;



    TKrayEventCreateTextStream          = record
                                            textStreamHandle: pointer;
                                            _fill_2         : pointer;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventDestroyTextStream         = record
                                            textStreamHandle: pointer;
                                            _fill_2         : pointer;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventNewLine                   = record
                                            textStreamHandle: pointer;
                                            _fill_2         : pointer;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventText                      = record
                                            textStreamHandle: pointer;
                                            textBuffer      : PKrayEventTextBufferData;
                                            text            : pansichar;
                                          end;
    TKrayEventGetText                   = record
                                            textStreamHandle: pointer;
                                            textBuffer      : PKrayEventTextBufferData;
                                            text            : pansichar;
                                          end;
    TKrayEventAllocTextBuffer           = record
                                            textStreamHandle: pointer;
                                            textBuffer      : PKrayEventTextBufferData;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventFreeTextBuffer            = record
                                            textStreamHandle: pointer;
                                            textBuffer      : PKrayEventTextBufferData;
                                            _fill_3         : pointer;
                                          end;
    TKrayEventGetTextProperty           = record
                                            textStreamHandle: pointer;
                                            propertyValue   : size_t;
                                            _fill_3         : pointer;
                                          end;



  type
    PKrayEventData  = ^TKrayEventData;
    TKrayEventData  = record
                        _type   : TKrayEventType;
                        instance: TKrayInstanceHandle;
                        userData: pointer;
                        case  integer  of
                           0: (
                                 union1: TKrayEventData1;
                                 union2: TKrayEventData2;
                                 union3: TKrayEventData3
                              );
                           1: (dataSetPixelArray            : TKrayEventSetPixelArray);
                           2: (dataSetPixelRectangle        : TKrayEventSetPixelRectangle);
                           3: (dataCreateTextStream         : TKrayEventCreateTextStream);
                           4: (dataDestroyTextStream        : TKrayEventDestroyTextStream);
                           5: (dataText                     : TKrayEventText);
                           6: (dataNewLine                  : TKrayEventNewLine);
                           7: (dataGetText                  : TKrayEventGetText);
                           8: (dataGetTextProperty          : TKrayEventGetTextProperty);
                           9: (dataCreateWindow             : TKrayEventCreateWindow);
                          10: (dataDestroyWindow            : TKrayEventDestroyWindow);
                          11: (dataResizeWindow             : TKrayEventResizeWindow);
                          12: (dataSetWindowTitle           : TKrayEventSetWindowTitle);
                          13: (dataSetWindowProperty        : TKrayEventSetWindowProperty);
                          14: (dataGetWindowProperty        : TKrayEventGetWindowProperty);
                          15: (dataGetWindowMousePosition   : TKrayEventGetWindowMousePosition);
                          16: (dataGetWindowMouseRawPosition: TKrayEventGetWindowMouseRawPosition);
                          17: (dataGetWindowKeyEvent        : TKrayEventGetWindowKeyEvent);
                          18: (dataAllocTextBuffer          : TKrayEventAllocTextBuffer);
                          19: (dataFreeTextBuffer           : TKrayEventFreeTextBuffer);
                      end;


  type
    TKrayEventHandler = procedure (const handler: TKrayEventData); cdecl;



implementation




  end.
