unit kray3loader;

interface

uses
  dynlibs,strings;

type
  KrayInstanceHandle = Pointer;
  KrayProtoHandle = Pointer;

  KrayEventData = Pointer;
  KrayEventHandler = Pointer;

var
  KrayGetInfoString: function(mode: Integer): PChar; {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayGetInfoInt: function(mode: Integer): Integer; {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayGetHostVersion: function(): Integer; {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayGetLibApiVersion: function(): Integer; {$IFDEF WINDOWS} cdecl; {$ENDIF}

  KrayLibraryCreateInstance: function(): KrayInstanceHandle; {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayLibraryReleaseInstance: procedure(instance: KrayInstanceHandle); {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayLibraryCreateInstanceWithEventHandler: function(): KrayInstanceHandle; {$IFDEF WINDOWS} cdecl; {$ENDIF}

  KrayInstanceParseExt: function(instance: KrayInstanceHandle; krayScript: PChar; optionalKrayScriptName: PChar): Integer; {$IFDEF WINDOWS} cdecl; {$ENDIF}

  KrayCreateDirectProto: function(instance: KrayInstanceHandle): KrayProtoHandle {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayReleaseProto: procedure(proto: KrayProtoHandle); {$IFDEF WINDOWS} cdecl; {$ENDIF}
  KrayInstanceSendBackEvent: function(): KrayInstanceHandle; {$IFDEF WINDOWS} cdecl; {$ENDIF}

function KrayLibraryLoad(libraryDir: PChar): Boolean;
function KrayInstanceParse(instance: KrayInstanceHandle; krayScript: PChar): Integer;
procedure KrayLibraryUnload();

implementation

type KrayLibraryHandle = Thandle;

var
   KrayLibrary: KrayLibraryHandle=KrayLibraryHandle(0);

   KrayLibraryLoadInternal: function(libraryDir: PChar): Integer; {$IFDEF WINDOWS} cdecl; {$ENDIF}

function getFn(name: PChar): Pointer;
begin
  if KrayLibrary <> 0 then
  begin
    getFn:=GetProcedureAddress(KrayLibrary, name);
    if getFn = Pointer(0) then KrayLibraryUnload();
  end;
end;

function KrayLibraryLoad(libraryDir: PChar): Boolean;  { call before any other Kray lib calls, and make sure it returns non-zero value }

const dllName : PChar = 'kray3c'
{$IFDEF WINDOWS}
 +'_'
 {$IFDEF WIN64}
  +'64'
 {$ELSE}
  +'32'
 {$ENDIF}
{$ENDIF}
+'.'+SharedSuffix;

var libName: PChar;

begin
  if KrayLibrary=0 then
  begin
    libName:=stralloc(length(libraryDir)+length(dllName)+1);
    strcopy(strecopy(libName,libraryDir),dllName);

    KrayLibrary:=SafeLoadLibrary(dllName);

    strdispose(libName);
  end;

  pointer(KrayLibraryLoadInternal):=getFn('KrayLibraryLoad');
  pointer(KrayLibraryCreateInstance):=getFn('KrayLibraryCreateInstance');
  pointer(KrayLibraryReleaseInstance):=getFn('KrayLibraryReleaseInstance');
  pointer(KrayInstanceParseExt):=getFn('KrayInstanceParse');
  pointer(KrayCreateDirectProto):=getFn('KrayCreateDirectProto');
  pointer(KrayReleaseProto):=getFn('KrayReleaseProto');
  pointer(KrayLibraryCreateInstanceWithEventHandler):=getFn('KrayLibraryCreateInstanceWithEventHandler');
  pointer(KrayInstanceSendBackEvent):=getFn('KrayInstanceSendBackEvent');
  pointer(KrayGetInfoString):=getFn('KrayGetInfoString');
  pointer(KrayGetInfoInt):=getFn('KrayGetInfoInt');
  pointer(KrayGetHostVersion):=getFn('KrayGetHostVersion');
  pointer(KrayGetLibApiVersion):=getFn('KrayGetLibApiVersion');

  if KrayLibrary<>0 then
  begin
    KrayLibraryLoad:= KrayLibraryLoadInternal(libraryDir)<>0
  end
  else
  begin
    KrayLibraryLoad:= false;
  end;
end;

function KrayInstanceParse(instance: KrayInstanceHandle; krayScript: PChar): Integer;
begin
  KrayInstanceParse:=KrayInstanceParseExt(instance, krayScript, 'Kray3forPascalCall');
end;

procedure KrayLibraryUnload();  { call only when all Kray instances are released }
begin
     if KrayLibrary<>0 then
     begin
       FreeLibrary(KrayLibrary);
       KrayLibrary:=0;
     end;
end;

end.
