#include "ruby.h"

#include "../../kraysdk/kray.h"
#include "../../kraysdk/staticlink.h"
#include "../../kraysdk/utils/rotalloc.h"

/*
#Makefile generator for Ruby

require 'mkmf'

# get rid of unnecessary libstdc++
$defs.push("-DMEM_USE_STDC_ALLOCATOR")
$defs.push("-fno-rtti")
$defs.push("-fno-threadsafe-statics")
$defs.push("-fno-exceptions")

# make GCC lib static (less dependencies, almost the same file size)
$libs+="-static-libgcc"

create_makefile( 'Kray' , 'kraysdk' )
*/

typedef VALUE (*RubyFn)(...);

RotationalPointerAllocator<Kray::Instance> rubyInstanceHandles;

inline size_t RubyToId(VALUE i){
	return (size_t)NUM2INT(i);
}
inline VALUE IdToRuby(size_t i){
	return INT2NUM((int)i);
}
inline const char* RubyToStringOrNull(VALUE i){
	return RSTRING_LEN(i)?RSTRING_PTR(i):0;
}

inline Kray::Instance* getKrayInstanceFromHandle(VALUE instance){
	return rubyInstanceHandles.getPointer(RubyToId(instance)-1);
}

static VALUE isLoaded(VALUE self){
    return Kray::getLibrary().isLoaded()?Qtrue:Qfalse;
}
static VALUE load(VALUE self, VALUE directory){
	Kray::getLibrary().load(RubyToStringOrNull(directory));

	return isLoaded(self);
}
static VALUE getInfoString(VALUE self,VALUE mode){
	return rb_str_new2(Kray::getLibrary().getInfoString(NUM2INT(mode)));
}
static VALUE getInfoInt(VALUE self,VALUE mode){
	return INT2NUM(Kray::getLibrary().getInfoInt(NUM2INT(mode)));
}

static VALUE createInstance(VALUE self){
	Kray::Instance* i=Kray::getLibrary().createInstance();

	if (!i){
		return IdToRuby(0);
	}

	size_t handle=rubyInstanceHandles.alloc(i);

	VALUE instance=IdToRuby(handle+1);

	return instance;
}
static VALUE releaseInstance(VALUE self,VALUE instance){
	Kray::getLibrary().releaseInstance(getKrayInstanceFromHandle(instance));
	rubyInstanceHandles.free(RubyToId(instance)-1);

	if (!Kray::getLibrary().hasInstances()){
		Kray::getLibrary().release();
		rubyInstanceHandles.freeAll();
	}

    return Qnil;
}
static VALUE instanceParse(VALUE self,VALUE instance,VALUE krayScript){
	Kray::Instance* i=getKrayInstanceFromHandle(instance);

	if (i){
		return INT2NUM(getKrayInstanceFromHandle(instance)->parse(StringValueCStr(krayScript),"Ruby"));
	}else{
		return Qnil;
	}
}

#define RUBY_FN(x,args) rb_define_module_function(Kray,#x,(RubyFn)x,args)

extern "C" void Init_Kray(){
	VALUE Kray = rb_define_module( "Kray" );   

	RUBY_FN(isLoaded,0);
	RUBY_FN(load,1);
	RUBY_FN(getInfoString,1);
	RUBY_FN(getInfoInt,1);
	RUBY_FN(createInstance,0);
	RUBY_FN(releaseInstance,1);
	RUBY_FN(instanceParse,2);
}