#ifndef PURE_C_KRAY_HEADER_H
#define PURE_C_KRAY_HEADER_H

#include "../base/calling.h"
#include "../base/vmap.h"
#include "kray1x.h"

#define _KRAY_ALLOW_DEPRECATED

#ifdef __cplusplus
extern "C"{
#endif

// pure C prototypes here
char kray_load();		// load kray.dll, call before calling any other function, returns 0 if Kray load fails
int kray_getLibApiVersion();	// check lib API version (host API version is defined by KRAY_API_VERSION macro)
void kray_renderInit();	// always before calling parse
int  kray_parse(const char*);	// parse Kray script
void kray_render(int flags);		// render
void kray_renderCleanUp();	// after rendering
void kray_release();	// release kray.dll
const char* kray_getKrayVersionString();
const char* kray_getInfoString(int stringId);

KrayHandle kray_getObjectByName(const char *name,KrayHandleType type);
KrayObject kray_addTriangle(const float *points,const float *normals,const float *geometric_normal,KraySurfaceHandle surface,KrayBlendHandle blend,KrayTextureHandle clipmap,KrayObjectGroupHandle node,size_t vertex_map_poly_index/*,int lemgroup*/,int flags);
float* kray_getVertexMapEntry(KrayVertexMapStruct *vmap,size_t index);
void kray_getBitmapInfo(const KrayBitmapHandle bitmap,KrayBitmapInfo* binfo);

int kray_addPolyToMesh(KrayMeshHandle msh,size_t n,int sfid,size_t *indices,int group);

void kray_resetCallbacks();		// resets all callbacks to initial state

void kray_setRenderEventHandler(void (KRAYDLL_CALL_TYPE *handler)(struct _KrayEventData*));

void KRAYDLL_CALL_TYPE kray_backHandler(struct _KrayEventData* data);
void KRAYDLL_CALL_TYPE kray_nullHandler(struct _KrayEventData* data);

#ifdef _KRAY_ALLOW_DEPRECATED
void kray_set_newline_callback(void (*_newline)());	// go to new line callback (text output)
void kray_set_textline_callback(void (*_textline)(const char*,int len));	// replace current text line
void kray_set_setwindowsize_callback(void (*_setwindowsize)(int x,int y));	// kray wants to open a window of given size
void kray_set_setpixel_callback(void (*_setpixel)(int x,int y,int c));	// kray want to set pixel at x,y of color c (0xRRGGBB)
void kray_set_interrupt_callback(char (*_interrupt)());		// kray calls this from time to time to check if user does not want to abort rendering
void kray_set_copypixels_callback(void (*_copypixels)(unsigned int* a,int sx,int x1,int y1,int x2,int y2));	// copy pixels callback (default implementation above in KrayEventListener::copyPixels() ), if not set, default implementation is used
void kray_set_renderevent_callback(void (KRAYDLL_CALL_TYPE *handler)(struct _KrayRenderEventData*));
//void kray_set_renderevent_callback(void (*_renderevent)(struct _KrayEventData* data));	// misc events (like render end)
#endif

#ifdef __cplusplus
} // extern "C"
#endif

#endif