#ifndef KRAY_DOUBLE_H
#define KRAY_DOUBLE_H

#include "../base/parser.h"
#include "../base/varlen.h"
#include "../symbol/symbol.h"
#include "../buffer/cmd.h"
#include "vector.h"

namespace Kray{

class Double : public KrayDoubleCore{
public:
	Double(double value=0){
		v=value;
	}
	Double(const KrayDoubleCore& core){
		v=core.v;
	}
	
	KrayDtTypes dtType() const{
		return KrayDtDoubleType;
	}
	
	operator double() const{
		return v;
	}
};

class DoubleSymbol : public DatatypeSymbol{
	void registerSymbol();
public:
	DoubleSymbol(CommandBuffer& ktb,const char* name=0):DatatypeSymbol(ktb,name){
		registerSymbol();
	}	

	void directSet(const Double& v);
	void directGet(Double&) const;
};

class VarLenDouble : public VarLenTemplate<Double>{
protected:
	VarLenDouble(){
		count=0;
	}
	VarLenDouble(const Vector& vector){	// vector view creator
		count=(size_t)vector.dim();
		v=(Double*)vector.v;
	}

public:
	VarLenDouble(const double& v0){	// vector view creator, v0 is referenced, not copied!
		count=1;
		v=(Double*)&v0;
	}
};

template <size_t N> class VarLenDoubleFixed : public VarLenFixedTemplate<VarLenDouble,N>{
};

class VarLenDoubleDynamic : public VarLenDynamicTemplate<VarLenDouble>{
public:
	VarLenDoubleDynamic(){
	}
	VarLenDoubleDynamic(size_t size){
		resize(size);
	}
};

};

#endif