#include "bitarray.h"

#include <own/mathtbox.h>

void BitArray::alloc(size_t s){
	freeMem();
			
	if (s>0){
		table=allocMem(bytesSize(s));
		size=s;
	}
	
	if (table){
		clear();
	}else{
		size=0;
	}
}
void BitArray::resize(size_t nsize){
	BitArrayBaseType* ntable=allocMem(bytesSize(nsize));

	size_t msize=stmin(bytesSize(),bytesSize(nsize));
	
	size_t i;
	for (i=0 ; i<msize ; i++){
		ntable[i]=table[i];
	}
	
	msize=bytesSize(nsize);
	for (; i<msize ; i++){
		ntable[i]=0;
	}
	
	freeMem();

	table=ntable;
	size=nsize;
}
void BitArray::set(){
	size_t bytes=bytesSize();

	for (size_t a=0; a<bytes ; a++){
		table[a]=~0;
	}
}
void BitArray::clear(){
	size_t bytes=bytesSize();

	for (size_t a=0; a<bytes ; a++){
		table[a]=0;
	}
}
BitArray::BitArrayBaseType* BitArray::allocMem(size_t size) const{
	return allocateTable<BitArrayBaseType>(size);
}
size_t BitArray::ones() const{
	if (!size) return 0;

	size_t res=0;
	size_t bytes=bytesSize()-1;

	for (size_t a=0 ; a<bytes ; a++){
		if (table[a]==~0){
			res+=BitArrayBaseTypeBits;
		}else{
			res+=countOneBits(table[a]);
		}
	}

	size_t mask=bitMask(0,size%BitArrayBaseTypeBits+1);
	res+=countOneBits(table[bytes]&mask);

	return res;
}
void BitArray::freeMem(){
	freeTable(table);
	table=0;
	
	size=0;
}
size_t BitArray::findFirst(char rev,size_t start,size_t stop) const{
	if (!size) return ~0;
	
	size_t starti=start/BitArrayBaseTypeBits;
	size_t stopi=stop/BitArrayBaseTypeBits;

	size_t byte;
	size_t bit;
		
	if (starti!=stopi){
		// check first size_t in range

		byte=table[starti];
		size_t startb=start%BitArrayBaseTypeBits;

		for (bit=startb ; bit<BitArrayBaseTypeBits ; bit++){
			if (!_get(byte,bit,rev)){
				return starti*BitArrayBaseTypeBits+bit;
			}
		}
	}
	
	for (size_t bytei=starti+1 ; bytei<stopi ; bytei++){	// check full size_t's in range
		byte=table[bytei];
		if (byte!=~0){
			return _findFirst(byte,rev);
		}
	}

	// check last size_t in range
	byte=table[stopi];
	size_t stopb=stop%BitArrayBaseTypeBits;

	size_t startb=0;

	if (starti==stopi){
		startb=start%BitArrayBaseTypeBits;
	}

	for (bit=startb ; bit<stopb ; bit++){
		if (!_get(byte,bit,rev)){
			return stopi*BitArrayBaseTypeBits+bit;
		}
	}

	return ~0;
}