#ifndef BITARRAY_H
#define BITARRAY_H

#include <stddef.h>	// for 
#include <own/mathtbox.h>

#include "allocator.h"

class BitArray : public MemoryAllocator{
	typedef size_t BitArrayBaseType;
	static const size_t BitArrayBaseTypeBits=sizeof(BitArrayBaseType)*8;

protected:
	BitArrayBaseType* table;
	size_t size;	// size in bits
	
	static char _get(BitArrayBaseType v,size_t shift,char rev=0){
		char res=(v&(((BitArrayBaseType)1)<<shift))!=0;
		
		return rev?!res:res;
	};
	static size_t _findFirst(BitArrayBaseType v,char rev=0){
		return youngestOneBit(rev?~v:v);
	}
public:
	static size_t stmin(size_t a,size_t b){
		return (a<b)?a:b;
	}

public:
	BitArray():table(0),size(0){
	};
	BitArray(size_t size):table(0){
		alloc(size);
	}
	
	~BitArray(){
		freeMem();
	}

	void setTableMemory(void* mem,size_t bytesSize){
		table=(BitArrayBaseType*)mem;
		size=bytesSize*BitArrayBaseTypeBits;
	}

	char ok() const{
		return table!=0;
	}
	
	static size_t bytesSize(size_t size){
		return (size+BitArrayBaseTypeBits-1)/BitArrayBaseTypeBits;
	}
	size_t bytesSize() const{
		return bytesSize(size);
	}
	size_t getSize() const{
		return size;
	}
	void alloc(size_t s);
	void resize(size_t nsize);
	void set();
	void clear();

	BitArrayBaseType* allocMem(size_t size) const;
	void freeMem();
	
	size_t ones() const;
	size_t zeros() const{
		return size-ones();
	}
	
	void clear(size_t index){
		size_t shift=index%BitArrayBaseTypeBits;
		index/=BitArrayBaseTypeBits;
		table[index]&=(((BitArrayBaseType)1)<<shift)^((BitArrayBaseType)-1);
	}
	void set(size_t index,char value){
		if (value){
			set(index);
		}else{
			clear(index);
		}
	}
	void set(size_t index){
		size_t shift=index%BitArrayBaseTypeBits;
		index/=BitArrayBaseTypeBits;
		table[index]|=((BitArrayBaseType)1)<<shift;
	}
	char get(size_t index) const{
		if (index>=size){
			return 0;
		}
		
		size_t shift=index%BitArrayBaseTypeBits;
		index/=BitArrayBaseTypeBits;
		
		return _get(table[index],shift);
	}
	char getAndClear(size_t index){
		char r=get(index);

		clear(index);

		return r;
	}
	
	char operator[](size_t index) const{
		return get(index);
	}
	
	size_t findFirst(char rev,size_t start,size_t stop) const;
};

#endif

