#include "dl.h"
#include "textbuffer.h"

///////////////////////////////////////////////////////

#if !defined(_WINDOWS_LIKE_DLL) && !defined(_UNIX_LIKE_DLL) && !defined(_NO_LIKE_DLL)
# if defined(_WIN32_WINNT) || defined(WIN32)
#  define _WINDOWS_LIKE_DLL
# endif
#endif

#ifdef _WINDOWS_LIKE_DLL
# include <windows.h>

void* DllLoad(const char* filename,int flags){
	return LoadLibraryA(filename);
}
void* DllFindProcedure(void* handle,const char* name,int flags){
	return (void*)GetProcAddress((HMODULE)handle, name);
}
void DllUnload(void* handle){
	FreeLibrary((HMODULE)handle);
}

#elif defined(_UNIX_LIKE_DLL)
# include <dlfcn.h>

void* DllLoad(const char* filename,int flags){
	return dlopen(filename, RTLD_LAZY);
}
void* DllFindProcedure(void* handle,const char* name,int flags){
	return dlsym(handle, name);
}
void DllUnload(void* handle){
	dlclose(handle);
}

#else

# ifndef _NO_LIKE_DLL_
#  ifdef _MULTIPVERBOSE
#   pragma message ("### Dll model not defined, Use _WINDOWS_LIKE_DLL or _UNIX_LIKE_DLL preprocessor flag")
#  endif
# endif

void* DllLoad(const char* filename,int flags){
	return 0;
}
void* DllFindProcedure(void* handle,const char* name,int flags){
	return 0;
}
void DllUnload(void* handle){
}

#endif

///////////////////////////////////////////////////////

void* DllLoadCombinedFilename(const char* directory,const char* filename,int flags,const char* extension,const char* sub1,const char* sub2,const char* sub3){
	ResizableBuffer fullName;

	if (directory){
		fullName.appendString(directory);
	}
	if (filename){
		fullName.appendString(filename);
	}
	if (sub1){
		fullName.appendString(sub1);
	}
	if (sub2){
		fullName.appendString(sub2);
	}
	if (sub3){
		fullName.appendString(sub3);
	}
	if (extension){
		fullName.appendString(extension);
	}

	fullName.append(0);

	return DllLoad(fullName.getBuffer(),flags);
}

void* DllLoadDebugOrReleaseVersion(const char* directory,const char* filename,const char* extension,int flags){
	void* hLib=0;

#if defined(_DEBUG)
	if (!hLib){
		hLib=DllLoadCombinedFilename(directory,filename,flags,extension,DllBusWidthSuffix(),"d");
	}
#endif // _DEBUG

	if (!hLib){
		hLib=DllLoadCombinedFilename(directory,filename,flags,extension,DllBusWidthSuffix());
	}

#if !defined(_DEBUG)
	if (!hLib){
		hLib=DllLoadCombinedFilename(directory,filename,flags,extension,DllBusWidthSuffix(),"d");
	}
#endif // !_DEBUG

	return hLib;
}
