#ifndef DYNAMIC_LINK_H
#define DYNAMIC_LINK_H

#if !defined(_WINDOWS_LIKE_DLL) && !defined(_UNIX_LIKE_DLL) && !defined(_MACOS_LIKE_DLL) && !defined(_NO_LIKE_DLL)
# if defined(_WINDOWS) || defined(_WIN32) || defined(WIN32) || defined(_WIN32_WINNT)
#  define _WINDOWS_LIKE_DLL
# elif defined(__APPLE__)
#  define _MACOS_LIKE_DLL
# elif defined(_LINUX)
#  define _UNIX_LIKE_DLL
# endif
#endif

#ifdef _MACOS_LIKE_DLL
# ifndef _UNIX_LIKE_DLL
#  define _UNIX_LIKE_DLL
# endif
# if defined(__i386__) || defined(__x86_64__)
#  ifndef _INTEL_CPU_DLL
#   define _INTEL_CPU_DLL
#  endif
# endif
#endif

#ifdef _WINDOWS_LIKE_DLL
# define DLLCURRENTDIR ""
# define DLLSLASH "\\"
# define DLLROOTDIR "..\\"
# define DLLDEFAULTEXTENSION ".dll"
  
# ifndef _INTEL_CPU_DLL
#  define _INTEL_CPU_DLL
# endif
#endif

#ifdef _UNIX_LIKE_DLL
# define DLLSLASH "/"
# define DLLCURRENTDIR "./"
# define DLLROOTDIR "../"
# if defined(_MACOS_LIKE_DLL)
#  define DLLDEFAULTEXTENSION ".dylib"
# else
#  define DLLDEFAULTEXTENSION ".so"
# endif
#endif

#ifndef DLLSLASH
# ifndef _NO_LIKE_DLL
#  define _NO_LIKE_DLL
# endif
# define DLLSLASH "/"
# define DLLCURRENTDIR ""
# define DLLROOTDIR "../"
# define DLLDEFAULTEXTENSION ""
#endif

#ifdef __cplusplus
inline const char DllBit64(){
	return (sizeof(void*)==8);
}
inline const char* DllBusWidthSuffix(){
#if defined(_MACOS_LIKE_DLL) && !defined(_MACOS_BUS_WIDTH_SUFFIX)
	return 0;
#else
	return DllBit64()?"_64":"_32";
#endif
}

extern "C"{
	void* DllLoad(const char* filename,int flags=0);
	void* DllFindProcedure(void* handle,const char* name,int flags=0);
	void* DllLoadCpuVersion(const char* directory,const char* filename,const char* extension,int flags=0,const char* defaultSuffix="fpu");
	void* DllLoadDebugOrReleaseVersion(const char* directory,const char* filename,const char* extension,int flags=0);
	void* DllLoadCombinedFilename(const char* directory,const char* filename,int flags=0,const char* extension=0,const char* sub1=0,const char* sub2=0,const char* sub3=0);
#else	// __cplusplus
	void* DllLoad(const char* filename,int flags);
	void* DllFindProcedure(void* handle,const char* name,int flags);
	void* DllLoadCpuVersion(const char* filename,const char* extension,int flags);
#endif	// __cplusplus
	void DllUnload(void* handle);
#ifdef __cplusplus
}
#endif

#if defined(_NO_LIKE_DLL)
# define DLLEXPORTFUNCTIONPREFIX extern "C"
#elif defined(_MSC_VER)
# define DLLEXPORTFUNCTIONPREFIX extern "C" __declspec(dllexport)
#else
# define DLLEXPORTFUNCTIONPREFIX extern "C" __attribute__ ((visibility("default"))) 
#endif

#endif
