#define JAVAVMPOINTER javaVM

#include "../../kraysdk/wrappers/java.h"
#include "../../kraysdk/staticlink.h"
#include "../../kraysdk/kray.h"

#include "../../kraysdk/base/listener.h"
#include "../../kraysdk/base/event.h"
#include "../../kraysdk/base/mt.h"
#include "../../kraysdk/base/bmp.h"
#include "../../kraysdk/base/bmpcall.h"
#include "../../kraysdk/base/rt.h"
#include "../../kraysdk/base/prscall.h"
#include "../../kraysdk/base/getinfo.h"
#include "../../kraysdk/buffer/dynamic.h"
#include "../../kraysdk/utils/str2num.h"
#include "../../kraysdk/base/dispatchlistener.h"
#include "../../kraysdk/utils/rotalloc.h"

// tester run config
// CMDP: E:\Use\Coding\JDK\jdk1.7.0\bin\java
// ARGS: -Djava.library.path=D:\Sources\C++\Kray\dll_workspace\ -Xss4m -cp d:\Sources\Java\KrayJni\build\classes com.kraytracing.Kray3JniTester

using namespace Kray;

JavaVM* javaVM=0;

RotationalPointerAllocator<Instance> javaInstanceHandles(1);

void KRAYDLL_CALL_TYPE JavaThreadInitializer(void*){
	if (javaVM){
		JNIEnv* r;

#ifdef JAVAJNI_ACT_VOID
		javaVM->AttachCurrentThread((void**)&r,0);
#else
		javaVM->AttachCurrentThread(&r,0);
#endif
	}
}
void KRAYDLL_CALL_TYPE JavaThreadDestroyer(void*){
	if (javaVM){
		javaVM->DetachCurrentThread();
	}
}
DECLAREJNI(void,threadFunction,com_kraytracing_jni_ThreadSpawner)(JNIEnv* env,jobject jthis,jlong f,jlong argument){
	typedef void (*fnPtr)(void* argument);

	((fnPtr)f)((void*)argument);
}

inline Instance* getInstanceFromHandle(jint instanceHandle){
	return *javaInstanceHandles.getData(instanceHandle);
}	

jint KrayRegisterInstance(Instance* i){
	getLibrary().vtable().kmt->setThreadCallbacks(i->getKrayInstanceHandle(),JavaThreadInitializer,JavaThreadDestroyer,(void*)0xfeedbac);

	return (jint)javaInstanceHandles.allocNonZero(i);
}

KRAYJNI(jint,KrayLibraryLoad)(JNIEnv* env,jobject jthis,jstring jLibDir){
	env->GetJavaVM(&javaVM);

	jint r=getLibrary().load(JNISTRING(jLibDir));

	getLibrary().setPlatformOption("JavaJM",0,javaVM);

	return r;
}
KRAYJNI(jint,KrayLibraryCreateInstance)(JNIEnv* env,jobject jthis){
	return KrayRegisterInstance(getLibrary().createInstance());
}
KRAYJNI(jint,KrayLibraryCreateInstanceWithListener)(JNIEnv* env,jobject jthis,jobject listner){
	JavaEventListener* jel=new JavaEventListener(listner,env);

	if (!jel->ok()){
		delete jel;
		jel=0;
	}

	if (!jel) return 0;

	return KrayRegisterInstance(getLibrary().createInstanceWithListner(jel));
}
KRAYJNI(void,KrayLibraryReleaseInstance)(JNIEnv* env,jobject jthis,jint instanceHandle){
	Instance* instance=getInstanceFromHandle(instanceHandle);

	if (!instance) return;

	EventListener* el=instance->getEventListener();

	getLibrary().releaseInstance(instance);
	javaInstanceHandles.free(instanceHandle);

	delete el;

	if (!getLibrary().hasInstances()){
		getLibrary().release();
	}
}
KRAYJNI(jint,KrayInstanceParse)(JNIEnv* env,jobject jthis,jint instanceHandle,jstring krayScript,jstring optionalKrayScriptName){
	Instance* instance=getInstanceFromHandle(instanceHandle);

	if (!instance) return 0;

	return instance->parse(JNISTRING(krayScript),JNISTRING(optionalKrayScriptName));
}

///////////////////////////////

JavaTextStreamEventListner::JavaTextStreamEventListner(JavaEventListener* parent,JNIEnv* env,jobject javaListner):parent(parent),jListener(javaListner,env){
	jclass lClass=env->GetObjectClass(jListener);

	mTextLine=env->GetMethodID(lClass,"textLine","(Ljava/lang/String;)V");
	mTextNewLine=env->GetMethodID(lClass,"textNewLine","()V");
	mGetTextFlags=env->GetMethodID(lClass,"getTextFlags","()I");
	mGetTextLine=env->GetMethodID(lClass,"getTextLine","(Ljava/lang/String;)Ljava/lang/String;");
}
void JavaTextStreamEventListner::textLine(const char* txt){
	JNIEnv* env=jListener.getEnv();

	env->CallVoidMethod(jListener,mTextLine,JAVASTRING(txt));
}

void JavaTextStreamEventListner::textNewLine(){
	JNIEnv* env=jListener.getEnv();

	env->CallVoidMethod(jListener,mTextNewLine);
};

int JavaTextStreamEventListner::getTextFlags(){
	JNIEnv* env=jListener.getEnv();

	return env->CallIntMethod(jListener,mGetTextFlags);
}

void JavaTextStreamEventListner::getTextLine(const char* promptString,KrayEventTextBufferData* bufferData){
	JNIEnv* env=jListener.getEnv();

	jstring line=(jstring)env->CallObjectMethod(jListener,mGetTextLine,JAVASTRING(promptString));

	if (line){
		jsize lineSize=env->GetStringUTFLength(line)+1;
	
		JniStringConverter lineUtf(line,env);

		if ((size_t)lineSize>bufferData->bufferSize){
			bufferData->bufferSize=lineSize;

			parent->allocTextBuffer(bufferData);
		}

		s2n_strcpy(bufferData->buffer,lineUtf);
	}else{
		bufferData->buffer=0;
	}
}
TextStreamListener* JavaEventListener::createTextListner(){
	JNIEnv* env=jListener.getEnv();

	jclass lClass=env->GetObjectClass(jListener);
	jmethodID mid=env->GetMethodID(lClass,"createTextListener","()Lcom/kraytracing/kray/TextStreamListener;");

	if (mid){
		jobject textListener=env->CallObjectMethod(jListener,mid);

		if (textListener){	// if null, listener failed to create
			JavaTextStreamEventListner* jtsel=new JavaTextStreamEventListner(this,env,textListener);

			if (!jtsel->ok()){
				delete jtsel;
				jtsel=0;
			}

			if (jtsel){
				return jtsel;
			}
		}
	}

	return 0;
}


#if 0
void javaVmLoader(){
	JavaVM* jvm;       /* denotes a Java VM */
	JNIEnv* env;       /* pointer to native method interface */
	JavaVMInitArgs vm_args; /* JDK/JRE 6 VM initialization arguments */
	
	const int nOptions=1;

	JavaVMOption* options = new JavaVMOption[nOptions];
	options[0].optionString = "";
	options[0].extraInfo=0;

	vm_args.version = JNI_VERSION_1_6;
	vm_args.nOptions = nOptions;
	vm_args.options = options;
	vm_args.ignoreUnrecognized = JNI_TRUE;

	/* load and initialize a Java VM, return a JNI interface
		* pointer in env */
	JNI_CreateJavaVM(&jvm, (void**)&env, &vm_args);
	delete[] options;

	/* invoke the Main.test method using the JNI */
	jclass cls = env->FindClass("Main");
	jmethodID mid = env->GetStaticMethodID(cls, "test", "(I)V");
	env->CallStaticVoidMethod(cls, mid, 100);
	/* We are done. */
	jvm->DestroyJavaVM();
}

#endif

/*
class start{
public:
	start(){
//		javaVmLoader();
	}
};

start s;
*/