#include "../../../kraysdk/utils/str2num.h"
#include "../../../kraysdk/legacy/kray1x_pure_c.h"
#include "../../../kraysdk/legacy/kray1x.h"

// Kray API

char kray_load(){
	return kray.load();
}
int kray_getLibApiVersion(){
	return kray.getLibApiVersion();
}
const char* kray_getKrayVersionString(){
	return kray->getKrayVersionString();
}
const char* kray_getInfoString(int string){
	return kray->getInfoString(string);
}
void kray_release(){
	kray.release();
}
void kray_renderInit(){
	kray->renderInit();
}
int kray_parse(const char* command){
	return kray.parse(command);
}
void kray_render(int flags=0){
	kray->render(flags);
}
void kray_renderCleanUp(){
	kray->renderCleanUp();
}
void kray_getBitmapInfo(const KrayBitmapHandle bitmap,KrayBitmapInfo* binfo){
	kray->getBitmapInfo(bitmap,binfo);
}
KrayHandle kray_getObjectByName(const char* name,KrayHandleType type){
	return kray->getObjectByName(name,type);
}
KrayObject kray_addTriangle(const float* points,const float* geometric_normal,const float* normals,KraySurfaceHandle surface,KrayBlendHandle blend,KrayBlendHandle clipmap,KrayObjectGroupHandle node,size_t vertex_map_poly_index/*,int lemgroup*/,int flags){
	return kray->addTriangle(points,normals,geometric_normal,surface,blend,clipmap,node,vertex_map_poly_index/*,lemgroup*/,flags);
}
float* kray_getVertexMapEntry(KrayVertexMapStruct* vmap,size_t index){
	if (!vmap) return 0;

	return (float*)(vmap->data)+vmap->dim*vmap->samples*index;
}
int kray_addPolyToMesh(KrayMeshHandle msh,int n,int sfid,size_t* indices,int group){
	return kray->addPolyToMesh(msh,n,sfid,indices,group);
}
void kray_setRenderEventHandler(void (KRAYDLL_CALL_TYPE *handler)(struct _KrayEventData*)){
	kray.setEventHandler((KrayRenderCallbackFunction)handler);
}
void KRAYDLL_CALL_TYPE kray_backHandler(struct _KrayEventData* data){
	return kray->sendBackEvent(*(Kray::EventData*)data);
}
void kray_resetCallbacks(){		// resets all callbacks to initial state
	kray.setEventListener();
}

// callbacks set functions

#ifdef _KRAY_ALLOW_DEPRECATED
class KrayPureCEventWrapper : public Kray2DeprecatedListner{
public:
	void (*_renderevent)(struct _KrayRenderEventData *);
	void (*_newline)();
	void (*_textline)(const char*,int len);
	void (*_setwindowsize)(int x,int y);
	void (*_setpixel)(int x,int y,int c);
	void (*_copypixels)(unsigned int* pixels,int sx,int x1,int y1,int x2,int y2);
	char (*_interrupt)();
	
public:
	void clear(){
		_renderevent=0;
		_newline=0;
		_textline=0;
		_setwindowsize=0;
		_setpixel=0;
		_interrupt=0;
		_copypixels=0;
	}
	void textLine(const char* txt){
		if (isEventString(KRAY_EVENT_RENDER_FINISHED,txt)){
			renderDone();
		}
		if (_textline){
			_textline(txt,(int)s2n_strlen(txt));
		}
	}
	void newLine(){
		if (_newline){
			_newline();
		}
	};
	void setWindowSize(int x,int y){
		if (_setwindowsize){
			_setwindowsize(x,y);
		}
	}
	void setPixel(int x,int y,int c){
		if (_setpixel){
			_setpixel(x,y,c);
		}
	}
	void renderDone(){
		if (_renderevent){
			struct _KrayRenderEventData kred;
			kred.type=KrayRenderEventDone;
			
			_renderevent(&kred);
		}
	};
	char interrupt(){
		if (_interrupt){
			return _interrupt();
		}
		return 0;
	}
	void copyPixels(unsigned int* pixels,int sx,int x1,int y1,int x2,int y2){
		if (_copypixels){
			_copypixels(pixels,sx,x1,y1,x2,y2);
		}else{
			Kray2DeprecatedListner::copyPixels(pixels,sx,x1,y1,x2,y2);
		}
	}
	
public:
	KrayPureCEventWrapper(){
		clear();
	}
};

static KrayPureCEventWrapper c_events;

void kray_set_newline_callback(void (*_newline)()){	// go to new line callback (text output)
	kray.setEventListener(&c_events);
	c_events._newline=_newline;
}
void kray_set_textline_callback(void (*_textline)(const char*,int len)){	// replace current text line
	kray.setEventListener(&c_events);
	c_events._textline=_textline;
}
void kray_set_setwindowsize_callback(void (*_setwindowsize)(int x,int y)){	// kray wants to open a window of given size
	kray.setEventListener(&c_events);
	c_events._setwindowsize=_setwindowsize;
}
void kray_set_setpixel_callback(void (*_setpixel)(int x,int y,int c)){	// kray want to set pixel at x,y of color c (0xRRGGBB)
	kray.setEventListener(&c_events);
	c_events._setpixel=_setpixel;
}
void kray_set_interrupt_callback(char (*_interrupt)()){		// kray calls this from time to time to check if user does not want to abort rendering
	kray.setEventListener(&c_events);
	c_events._interrupt=_interrupt;
}
void kray_set_copypixels_callback(void (*_copypixels)(unsigned int* array,int sx,int x1,int y1,int x2,int y2)){
	kray.setEventListener(&c_events);
	c_events._copypixels=_copypixels;
}
void kray_set_renderevent_callback(void (*_renderevent)(struct _KrayRenderEventData* eventData)){
	kray.setEventListener(&c_events);
	c_events._renderevent=_renderevent;
}
#endif
