#ifndef SDK_KRAY_JNI_H
#define SDK_KRAY_JNI_H

/*
	Kray SDK Java JNI wrapper header
*/

#include <jni.h>

#define KRAYJNI(x,name) extern "C" JNIEXPORT x JNICALL Java_com_kraytracing_kray_Library_##name

#define JNISTRING(string) JniStringConverter(env,string)
#define JAVASTRING(string) (jstring)JniLocalAutoString(env,string)

class JniEnvHolder{
protected:
	JNIEnv* env;

public:
	JniEnvHolder(JNIEnv* env):env(env){
	}

	JNIEnv* getEnv() const;
};

class JniStringConverter : public JniEnvHolder{
	jstring js;
	const char* cs;

public:
	JniStringConverter(JNIEnv* env,jstring string):JniEnvHolder(env),js(string){
		cs=env->GetStringUTFChars(js,0);
	}
	~JniStringConverter(){
		env->ReleaseStringUTFChars(js,cs);
	}

	operator const char*() const{
		return cs;
	}
};

class JniLocalAutoObject : public JniEnvHolder{
protected:
	jobject localObject;

public:
	JniLocalAutoObject(JNIEnv* env,jobject j):JniEnvHolder(env),localObject(j){
	}
	~JniLocalAutoObject(){
		if (localObject){
			env->DeleteLocalRef(localObject);
		}
	}

	operator jobject() const{
		return (jobject)localObject;
	}
};

class JniLocalAutoString : public JniEnvHolder{
public:
	jstring string;

public:
	JniLocalAutoString(JNIEnv* env,const char* txt):JniEnvHolder(env){
		string=env->NewStringUTF(txt);
	}
	~JniLocalAutoString(){
		if (string){
			env->DeleteLocalRef(string);
		}
	}

	operator jstring() const{
		return string;
	}
};

class JniGlobalReference : public JniEnvHolder{
//	JNIEnv* env;
	jobject reference;

public:
	JniGlobalReference(JNIEnv* env,jobject obj);
	~JniGlobalReference();

//	JNIEnv* getEnv() const{
//		return env;
//	}

	operator jobject() const{
		return reference;
	}
};

#endif
